/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */

package com.funambol.mailclient.ui.utils;

import com.funambol.util.Log;
import com.funambol.mailclient.ui.controller.Theme;

import java.io.InputStream;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Font;

import net.rim.blackberry.api.homescreen.HomeScreen;
import net.rim.device.api.system.Bitmap;

/**
 * This class extends the basic user notification mechanism. The purpose of this
 * class is to implement the mechanism for changing the application icon.
 * This is possible on BlackBerry through the RIM specific APIs.
 * This class can only be compiled if RIM apis are available and it cannot be
 * shipped to any system other then BB.
 */
 
public class BlackBerryUserNotificationManager extends UserNotificationManager {

    // Current displayed icon:
    private static final int DEFAULT_ICON   = 0;
    private static final int NEW_EMAIL_ICON = 1;

    private static int currentIcon        = DEFAULT_ICON;
    private static int currentNewMessages = -1;

    protected BlackBerryUserNotificationManager() {}

    public void showNewEmailIcon() {
        showNewEmailIcon(-1);
    }
    
    public void showNewEmailIcon(int newMessages) {
        if (newMessages != -1) {
            changeTitle("Funambol (" + newMessages + ")");
        } else {
            changeTitle("Funambol  (new messages)");
        }

        if (currentIcon != NEW_EMAIL_ICON || newMessages != currentNewMessages) {
            Image img = Theme.getICONNEWMSG_IMG();

            // This piece of code can write the number of message on the icon
            // or perform any icon manipulation. At the moment writing on the
            // icon leads to very poor results, as the Font is far too big for
            // an icon....
            /*
            int width  = img.getWidth();
            int height = img.getHeight();
            Image mutableImg = Image.createImage(width, height);
            int[] pixels = new int[width * height];
            img.getRGB(pixels, 0, width, 0, 0, width, height);
            Graphics g = mutableImg.getGraphics();

            String msg;
            if (newMessages != -1) {
                msg = ""+newMessages;
            } else {
                msg = "*";
            }
            Font font = Font.getFont(Font.FACE_PROPORTIONAL, Font.STYLE_PLAIN, Font.SIZE_SMALL);
            g.setFont(font);
            int msgWidth = font.stringWidth(msg);
            int msgHeight = font.getHeight();

            // Draw the icon
            g.drawRGB(pixels, 0, width, 0, 0, width, height, false);
            // Draw the messages counter
            g.setColor(0xFF0000); // Red
            g.fillRect(width - msgWidth - 2, 2, width -1, msgHeight + 2);
            g.setColor(0x000000); // Black
            g.drawString(msg, 0, 0, Graphics.TOP|Graphics.RIGHT);
            */

            setNewIcon(img);
            currentIcon = NEW_EMAIL_ICON;

            currentNewMessages = newMessages;
        }
    }

    public void hideNewEmailIcon() {
        if (currentIcon != DEFAULT_ICON) {
            Image img = Theme.getICON_IMG();
            setNewIcon(img);
            currentIcon        = DEFAULT_ICON;
            currentNewMessages = -1;
        }
    }

    private void changeTitle(String newName) {
        // This is supported only on OS 4.2...
        //HomeScreen.setName(newName);
    }

    private void setNewIcon(Image img) {
        Log.debug("[BlackBerryUserNotificationManager] Changing home screen icon");
        int[] argb = new int[img.getWidth()*img.getHeight()];
        img.getRGB(argb, 0, img.getWidth(), 0, 0, img.getWidth(),
                   img.getHeight());
        try {
            Bitmap bitmap = null;
            bitmap = new Bitmap(img.getWidth(), img.getHeight());
            bitmap.setARGB(argb, 0, img.getWidth(), 0, 0,
                           img.getWidth(), img.getHeight());
            HomeScreen.updateIcon(bitmap); 
        } catch (Exception e) {
            Log.error("[BlackBerryUserNotificationManager] Cannot create bitmap");
        }
    }
}

