/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient;

import java.io.IOException;
import java.util.Date;

import javax.microedition.io.ConnectionNotFoundException;
import javax.microedition.io.PushRegistry;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import com.funambol.mail.MessageFlags;
import com.funambol.mail.Store;
import com.funambol.mailclient.config.ConfigManager;
import com.funambol.mailclient.config.MailClientConfig;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.sm.SyncClient;
import com.funambol.mailclient.ui.controller.AlarmManager;
import com.funambol.mailclient.ui.controller.AppProperties;
import com.funambol.mailclient.ui.controller.Killable;
import com.funambol.mailclient.ui.controller.LifeCycleController;
import com.funambol.mailclient.ui.controller.MessageHandler;
import com.funambol.mailclient.ui.controller.Theme;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.controller.UIThreadPoolMonitor;
import com.funambol.mailclient.ui.view.ExitPopupAction;
import com.funambol.mailclient.ui.view.ModalPopup;
import com.funambol.mailclient.ui.view.SplashScreen;
import com.funambol.push.OTAService;
import com.funambol.util.Log;
import javax.microedition.io.HttpConnection;
import com.funambol.util.ConnectionManager;

//#ifdef isBlackberry
//# import com.funambol.mailclient.ui.controller.UIConnectionListener;
//# import com.funambol.storage.BlackberryRecordStore;
//# import com.funambol.util.ConnectionConfig;
//# import com.funambol.storage.*;
//#endif

public class Funambol extends MIDlet implements Killable, Runnable {
    
    private MailClientConfig mailClientConfig;
    private OTAService otaService;
    private MessageHandler messageHandler;
    private InitConfigManager cm;
    private SplashScreen splashScreen;
    private static boolean hasFinished = false;
    
    public Funambol() {
        //startAppProbe = System.currentTimeMillis();
        splashScreen = new SplashScreen();
        Display.getDisplay(this).setCurrent(splashScreen);
    }
    
    protected void startApp() {
        
       
        if (LifeCycleController.isPaused == false) {

            // init localization for specific language
            //Localization.setMessages(new IT_LocalizedMessages());

            // Initialize the ThreadPoolMonitor
            UIController.initThreadPool();
            UIController.getThreadPool().startThread(this).setPriority(Thread.MAX_PRIORITY);
            
        } else {

            // see finally block in run method to understand reason of hasFinished usage
            if (hasFinished) {
            LifeCycleController.isPaused = false;
            LifeCycleController.getInstance().resume();
        }

        }
    }
    
    public void setSplashPhase(String message, int iconNumber) {
        splashScreen.setInitPhase(message, iconNumber);
    }
    
    public void setSplashPhase(String[] message, int iconNumber) {
        
        splashScreen.setInitPhase(message, iconNumber);
    }
    
    private void startSync(final boolean ok2Sync) {
        Log.debug("[Funambol.startSync]Number of runnable threads: " + UIController.getThreadPool().getRunnableCount());
        if (ok2Sync) {
            if (!UIController.otaStarted) {
                if (mailClientConfig.isSyncOnStartupEnabled()) {
                    Log.info("starting sync");
                    UIController.sync(true);
                } else {
                    Log.info("sync on startup disabled");
                }
            } else {
                Log.info("[FunambolMIDlet] Ota started the application... sync using SAN");
                
                // Workaround to avoid startsync failing with fake SMS incoming:
                // i.e. Sprint MotoV3m
                try {
                    Thread.sleep(3000);
                } catch (InterruptedException e) {
                }
                if ((!SyncClient.getSyncClient().isBusy()) &&
                     (otaService != null && !otaService.smsHandling)) {
                    Log.info("Sprint fix - starting sync due to fake OTA start");
                    UIController.sync(true);
                }
                
            }
        }
    }
    
    public void destroyApp(boolean unconditional) {
        Log.debug("DestroyAPP");
        ModalPopup popup =
                new ModalPopup(Localization.getMessages().POPUP_EXIT_TITLE,
                Localization.getMessages().POPUP_EXIT_TEXT,
                new ExitPopupAction());
        UIController.showModalPopup(popup, UIController.getInboxMessageList());
        
    }
    
    /**
     * Pausing the app means to put the Controller in paused mode.
     * In this way it will perform all the operations needed later when resuming
     */
    protected void pauseApp() {
        
        LifeCycleController.getInstance().pause();
        
    }
    
    /**
     * Exit the application. The exit method is invoked by the Controller
     * to exit the MIDlet
     */
    public void exit() throws MIDletStateChangeException {
      
        if (otaService != null) {
            otaService.stopService();
        }
        
        //Call the CTPMAnager to stop the CTP service
        CTPManager.getInstance().stopCTP();
        
        //#ifdef isBlackberry
        // need to remove the saved config 'cause sometimes static classes
        // are not removed in bb
        //# if (!SyncClient.getSyncClient().isBusy()) {
        //#     Log.debug("[Funambol.exit]Removing configuration");
        //#     ConnectionConfig.removeSavedConfig();
        //#     Log.debug("[Funambol.exit]Resetting permission");
        //# }
        //# ConnectionConfig.resetConfigurationsPermissions();
        //#endif

        
        Log.debug("[Funambol.exit]Number of runnable threads: " + UIController.getThreadPool().getRunnableCount());
        
        notifyDestroyed();
    }
    
    public void run() {
        
        try {
            boolean ok2Sync = false;

            boolean upgraded = false;


            UIController.init(this);

            // Initialize Logging facility ASAP
            UIController.initLogger();
            Log.info("[FunambolMIDlet] Starting main thread");

            //#ifdef isBlackberry
            //# if (ConnectionConfig.isUserConfirmationNeeded()) {
            //#     Log.debug("[Funambol] need to use UIConnectionListener");
            //#     ConnectionManager.getInstance().setConnectionListener(new UIConnectionListener());
            //# }
            //#endif


            //UIController.startAppProbe = startAppProbe;
            UIController.startupFreeMem = Runtime.getRuntime().freeMemory();

            // Create the message handler which is responsible for handling push
            // notifications
            messageHandler = new MessageHandler();
            // This code is for backward compatibility. Migration from v6 to v6.5
            // Can be removed in future relase
            ConfigManager.upgrade();
        
            cm = new InitConfigManager(this);
            cm.initConfig();
        
            if (UIController.isFirstSync){
                checkForFunambolUpdate();
            }
        
            UIController.initMessageManager();
            // This code is for backward compatibility. Migration from v6 to v6.5
            // Can be removed in future relase
                upgraded = handleUpgrade();
        
            // loading opened and closed image, they'll be probably needed later and
            // this minimize the empty inbox for devices like nokia 6630
            Theme.getImageFromFlags(MessageFlags.OPENED);
            Theme.getImageFromFlags(0);


            ok2Sync = UIController.initInbox();
       
            cm.initListener();
        
            // Do not resync if we performed an upgrade
            startSync(ok2Sync && !upgraded);
        
            UIController.loadContacts();
        
            // If sync on startup is enabled, we wait for the sync authentication
            // to be completed. This allow the authentication to terminate before
            // starting CTP at the very first run.
            if (!mailClientConfig.isSyncOnStartupEnabled()) {
                CTPManager.getInstance().startCTP();
            }
        
        } finally {
            // this is neeeded for first run after the installation on BB devices
            // since the app is paused and resumed at the startup before the run
            // method has time to finish, so we use hasFinished to ignore resume
            // requests until we're out of this run method

            hasFinished = true;
        }


    }
    
    private void checkForFunambolUpdate(){
        final String autoUpdateUrl = UIController.appProperties.get(AppProperties.AUTOUPDATE_URL);

        if (autoUpdateUrl!=null) {
            new Thread() {
                public void run() {
                    try {
                        HttpConnection hc = (HttpConnection) ConnectionManager.getInstance().open(autoUpdateUrl);
                        hc.close();
                    } catch (Exception ex) {
                        Log.error("[Funambol] An update error occurred");	
                    }
                }
            }.start();
        }
    }

    /**
     * This method checks if an upgrade is needed. The current implementation
     * checks if the Store version is less than 102 and in this case it starts a
     * reset inbox to resynchronize the inbox.
     * The synchronization is executed in a different thread, therefore the
     * method may report a success even if the synchronization fails. On failure
     * the inbox could be partially filled (or even empty), but data would be
     * consistent between client and server.
     *
     * @return true on successful upgrade
     */
    private boolean handleUpgrade() {

        // Check if we need to finish upgrade
        if (mailClientConfig.getStoreVersion() < Store.VERSION_102) {
            // The store is still in the old format. We must perform a
            // reset inbox for the upgrade to be completed.
            String msg = Localization.getMessages().UPGRADE_REQUIRED;
            UIController.showModalAlert(msg);

            // Reset inbox
            try {
                // This is a non blocking operation, therefore the method exits
                // immediately. If the update is not performed properly (for
                // example because of no network connectivity) then the user is
                // left without any message.
                UIController.resetAccount();
                mailClientConfig.setStoreVersion(Store.LATEST_VERSION);
                ConfigManager.saveConfig();
                return true;
            } catch (Exception e) {
                Log.error("Error while resetting account: " + e.toString());
                return false;
            }

        } else {
            // No update is necessary
            return false;
        }

    }

    /**
     * @return the splashscreen.
     * this method is used by the threadpoolmonitor to ensure robustness
     * when catching exceptions thrown at startup
     */
    public Displayable getSplashScreen() {
        return splashScreen;
    }

    private class InitConfigManager {

        Funambol midlet;

        public InitConfigManager(Funambol midlet) {
            this.midlet = midlet;
        }

        private void initConfig() {

//            UIController.initThreadPool();

            //long startInitConfig = System.currentTimeMillis();
            // Mail Configuration initInbox
            // We check in Storage then in JAD and at the end load a default config
            try {
                mailClientConfig = ConfigManager.getConfig(midlet);
            } catch (Exception e) {
                Log.error("Invalid mail client configuration: " +
                        "creating a default configuration");
                mailClientConfig = new MailClientConfig();
                e.printStackTrace();
            } finally {
                UIController.mailClientConfig = mailClientConfig;

                if (mailClientConfig.isSchedulerEnabled()) {
                    setAlarm();
                } else {
                    AlarmManager.getInstance().cancelTimerTask();
                }
                Log.setLogLevel(mailClientConfig.getLogLevel());

            }
        }

        private void setAlarm() {
            int minutes = mailClientConfig.getPollInterval();
            // if pollInterval is 0 (previous releases) set 10 minutes
            minutes = (minutes <= 0) ? 10 : minutes;
            AlarmManager alarmManager = AlarmManager.getInstance();
            if (minutes >= 1440) {
                int days = (mailClientConfig.getPollInterval() / 60) / 24;
                alarmManager.startSyncTimerTask(
                        new Date(mailClientConfig.getNextTimeAlarm()), days);
            } else {
                alarmManager.startSyncTimerTask(minutes);
            }
            //mailClientConfig.enableScheduler(true);
            mailClientConfig.setPollInterval(minutes);
        //mailClientConfig.setNextTimeAlarm(
        //        alarmManager.getDateFromMinutes(minutes).getTime());
        }

        private void registerConnection(String port) {
            //Checking Connection for Push registry listener
            String connection = "sms://:" + port;
            String midletName = UIController.midlet.getClass().getName();

            Log.error("Connection: " + connection);
            Log.error("MIDlet Name: " + midletName);

            try {
                PushRegistry.registerConnection(connection, midletName, "*");
            } catch (IllegalArgumentException iae) {
                Log.error("connection string or filter string " +
                        "not valid: " + iae);
                iae.printStackTrace();
            } catch (ConnectionNotFoundException cfe) {
                Log.error("Runtime system does not support push delivery " +
                        "for the requested connection protocol: " + cfe);
                cfe.printStackTrace();
            } catch (IOException ioe) {
            //If a connection has already been registered, ignores last
            //statement
            } catch (ClassNotFoundException cnf) {
                Log.error("Runtime system does not support push delivery " +
                        "for the requested connection protocol: " + cnf);
                cnf.printStackTrace();
            } catch (SecurityException se) {
                Log.error("User didn't grant permission to push " +
                        "listener: " + se);
            //se.printStackTrace();
            }
        }

        /**
         * This method initializes SMS push listening. The push listening can be
         * disabled, in such a case the method simply returns and does nothing.
         */
        public void initListener() {

            Log.debug("[FunambolMIDlet] initializing SMS listeners");

            //String smsPush = getAppProperty("MIDlet-Push-1");
            String smsPush = UIController.appProperties.get(AppProperties.MIDLET_PUSH_1);
//#ifdef PushRuntime
//#         smsPush = "true";
//#endif
            if (smsPush == null) {
                Log.debug("[FunambolMIDlet] SMS listening disabled");
                return;
            }

            //String sanPort = getAppProperty("SAN-Port");
            String sanPort = UIController.appProperties.get(AppProperties.SAN_PORT);
            if (sanPort == null) {
                sanPort = "50001";
                Log.info("[FunambolMIDlet] SAN port not defined in JAD, using default " + sanPort);
            }
            /** Connections detected at start up. */
            try {

//#ifdef PushRuntime
//#             registerConnection(sanPort);
//#endif

                // List all the registered connections
                String[] connections = PushRegistry.listConnections(false);
                otaService =
                        new OTAService(sanPort,
                        UIController.getThreadPool());

                otaService.setPushNotificationListener(messageHandler);
                //OTAConfigListener =
                //new OTAMessagesListener(getAppProperty("OTAC-Port"));

                if (connections == null || connections.length == 0) {
                    Log.info("[FunambolMIDlet] Application not registered for incoming SMS");
                } else {
                    Log.info(connections.length + " connections registered.");
                    // list only the active registered connections
                    connections = PushRegistry.listConnections(true);

                    //OTAConfigListener.init();
                    otaService.startService();
                    if (connections == null || connections.length == 0) {
                        Log.info("[FunambolMIDlet] No active connections.");
                        Log.info("[FunambolMIDlet] User started the application.");
                    } else {
                        Log.info(connections.length + " active connections.");
                        UIController.otaStarted = true;
                        Log.info("[FunambolMIDlet] Registry connection found: " + connections[connections.length - 1]);
                        if (connections[connections.length - 1].endsWith(sanPort)) {
                            OTAService.startViaOTA = true;
                            Log.info("[FunambolMIDlet] SAN Message started the application.");
                        } else {
                            Log.info("[FunambolMIDlet] OTA Config Message started the application.");
                        }

                    }
                }
            } catch (Exception exp) {
                Log.error("Exception occurred listening for SMS: " + exp.getMessage());
                UIController.showErrorAlert(
                        Localization.getMessages().ALERT_SECURITY_EXCEPTION_MESSAGE);
                try {
                    Thread.sleep(3000);
                } catch (InterruptedException ex) {
                    Log.error(this, "interruptedException in Exception in " +
                            "initlistener() method ");
                    ex.printStackTrace();
                }
            }
        }
    }
}

