/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */


package com.funambol.mailclient.ui.controller;

import com.funambol.mailclient.config.ConfigException;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.sm.SyncClient;
import com.funambol.mailclient.ui.view.ContactList;
import com.funambol.mailclient.ui.view.InboxMessageList;
import com.funambol.util.Log;
import com.funambol.util.BasicSyncListener;
import com.funambol.util.SyncListener;
import com.funambol.syncml.protocol.SyncML;

/* Note:
 * In multisession mode (when we synchronize more repositories in the same
 * session), we assume that the MailSyncListener is the main listener.
 * The main listener is in charge of performing operations related to the
 * beginning/end of the entire session (such as enabling/disabling sync button).
 */
import java.io.IOException;

public class ContactSyncListener extends BasicSyncListener {

    private boolean multiSession;
   
    /** Creates a new instance of MessageListUpdater */
    public ContactSyncListener() {
        multiSession = false;
    }

    public void startSession() {

        Log.debug(this, "Start session");

        if (!multiSession) {
            //DISABLE GetMail Command during sync email
            UIController.enableSyncCommands(false);
        }
        // Update msg
        String msg = Localization.getMessages().SYNC_CONTACTS;
        UIController.updateInboxMessageList(msg);
    }

    public void endSession(int status) {

        Log.debug(this, "End session");

        if (!multiSession) {
            // Reset the title
            UIController.resetInboxTitle();
            //ENABLE GetMail Command at the end of sync email
            UIController.enableSyncCommands(true);

            ContactList contactList = UIController.getContactList();
            contactList.initializeContactList();
            contactList.setTitle(contactList.getDefaultTitle());
        }
    }
    
    public void endConnecting(int alertCode) {
        if (alertCode==SyncML.ALERT_CODE_REFRESH_FROM_SERVER) {
            UIController.getContactManager().resetContactList();
        }
    }

    /**
     * This method does not belong to the SyncListener interface.
     * A multi session is a concept introduced by the SyncClient which is able
     * to perform the sync of multiple repositories.
     *
     * @param repositories repositories to be synchronized
     */
    public void startMultiSession(boolean repositories[]) {
        multiSession = true;
    }

    /**
     * This method does not belong to the SyncListener interface.
     * A multi session is a concept introduced by the SyncClient which is able
     * to perform the sync of multiple repositories.
     *
     */
    public void endMultiSession() {

        multiSession = false;
    }
    
    public void endSyncing() {
        Log.debug("[CSL] ending sync!! UIC isFirstSync value=" + UIController.isFirstSync);
        if (UIController.isFirstSync) {
            UIController.isFirstSync=false;
            Log.debug("[CSL] ending sync!! Change Value to false ");
        }
    }
}
