/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.funambol.mailclient.ui.controller;

import com.funambol.mail.Store;
import com.funambol.mailclient.config.ConfigException;
import com.funambol.mailclient.config.ConfigManager;
import com.funambol.mailclient.config.MailClientConfig;
import com.funambol.storage.AbstractRecordStore;
import com.funambol.storage.Serializable;
import com.funambol.syncml.devinf.DevInf;
import com.funambol.syncml.devinf.Ext;
import com.funambol.util.CodedException;
import com.funambol.util.ConnectionManager;
import com.funambol.util.HttpTransportAgent;
import com.funambol.util.Log;
import com.funambol.util.SimpleStreamReader;
import com.funambol.util.StreamReader;
import com.funambol.util.StringUtil;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Vector;
import javax.microedition.io.HttpConnection;
import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreFullException;
import javax.microedition.rms.RecordStoreNotFoundException;

/**
 * 
 * this class fills the server capabilities
 * 
 */
public class ServerCapFiller {

    // we have not checked yet the status
    public static final int STAUS_UNCHEKED = 0;
    // checking
    public static final int STAUS_CHECKING = 1;
    // checked
    public static final int STAUS_CHECKED = 2;
    // check failed
    public static final int STAUS_CHECK_FAILED = 3;
    //the store name
    private static final String STORE_NAME = "ServerCapabilities";
    // config versions
    private static final int VERSION_100 = 100;
    // this is the current version
    private static final int VERSION = VERSION_100;
    // used to build the url with server capabilities
    private static final String SERVER_CAP_SUFFIX = "/info";
    // prefix for EXTs
    private static final String EXT = "Ext=";
    // the object with servercaps
    private static DevInf serverCapabilities = null;
    // the info url with servercaps
    private static String serverCapURL = null;
    //status
    private static int status = STAUS_UNCHEKED;


    //--------------------------------------------------------- constructors
    //private constructor to prevent this to be istantiated
    private ServerCapFiller() {
        ; //doing nothing here
    }

    //--------------------------------------------------------- public methods
    /**
     * delete all the saved capabilities from memory and persistent storage.
     */
    public static void deleteSavedConfig() {

        Log.info("Deleting server capabilities saved config");
        serverCapabilities = null;
        serverCapURL = null;
        status = STAUS_UNCHEKED;
        deleteStoredConfig();

    }

    /**
     * 
     * get the servercapabilities from storage or remote server
     * 
     * @return the server capabilities
     * @throws ConfigException if config has not been set
     */
    public static DevInf getServerCapabilities() throws ConfigException {

        

        try {
            // if we've not received yet the capabilities we need to get them 
            if (serverCapabilities == null) {
                status = STAUS_CHECKING;
                Log.info("reloadin server cap");
                // this fills servercapabilities
                reloadServerCap();
                status = STAUS_CHECKED;        
            }
            
        } catch (Exception e) {
            // we've had an exception, check of capabilities failed!
            Log.error("[ServerCapFiller.getServerCapabilities] error getting " +
                    "server cap: " + e + " moving to state CHECK_FAILED");
            status = STAUS_CHECK_FAILED;
            
            if (e instanceof CodedException) {
                throw (CodedException) e;
            } else if (e instanceof ConfigException ) {
                throw (ConfigException)e;
            } else {
                Log.error("Unknown exception type: " + e);
                e.printStackTrace();
            }
        }

        return serverCapabilities;

    }

    /**
     * tries to get the server capabilities from
     * the RMS, if they're not found it tries to get them from the
     * server
     * @throws configexception if config has not been set
     */
    public static boolean reloadServerCap() throws ConfigException, CodedException {

        if (!fillServerCapFromStorage()) {
            return fillServerCapFromServer();
        } else {
            return true;
        }

    }

    public static boolean isGettingServerCapabilities() {
        return status == STAUS_CHECKING;
    }

    /**
     * get current status of the class as one of
     * <ul>
     * <li>STAUS_UNCHEKED</li>
     * <li>STAUS_CHECKING</li>
     * <li>STAUS_CHECKED</li>
     * <li>STAUS_CHECK_FAILED</li>
     * </ul>
     * @return current staus
     */
    public static int getStatus() {
        
        Log.debug("[ServercapFiller.getStatus] returning status " + status);
        return status;
    }

    //--------------------------------------------------------- private methods
    /**
     * get server cap from server.
     */
    private static boolean fillServerCapFromServer() throws ConfigException, CodedException {

        Log.info("[ServerCapFiller] filling server cap from server");

        HttpTransportAgent ta = new HttpTransportAgent(getServerCapURL(), false, false);

        String response = ta.sendMessage("");
        parseCapabilities(response);
        saveCapabilities();
        return true;
    // }



    }

    private static void SafeCloseRecordStore(AbstractRecordStore s) {

        if (s != null) {
            try {
                s.closeRecordStore();
            } catch (RecordStoreException e) {
                //nothing to do here!
                ;
            }
        }
    }

    // this nukes the rms
    private static void deleteStoredConfig() {
        try {
            Log.info("nuking stored server capabilities config");
            AbstractRecordStore.deleteRecordStore(STORE_NAME);
        } catch (RecordStoreNotFoundException ex) {
        // nothing to do here, we're happy the record store was not there! :D
        } catch (RecordStoreException ex) {
            Log.error("[ServerCapFiller] error deleting recordstore " + ex);
            ex.printStackTrace();
        }
    }

    /**
     * write config version to the first record of the recordstore
     */
    private static void initRecordStore() {
        AbstractRecordStore s = null;
        try {

            // ensure we're initing an empty recordstore
            s = AbstractRecordStore.openRecordStore(STORE_NAME, true);
            if (s.getNumRecords() > 0) {
                s.closeRecordStore();
                deleteStoredConfig();
                s = s.openRecordStore(STORE_NAME, true);
            }

            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            DataOutputStream os = new DataOutputStream(baos);
            os.writeInt(VERSION);
            byte[] data = baos.toByteArray();
            s.addRecord(data, 0, data.length);

        } catch (IOException ex) {
            ex.printStackTrace();
        } catch (RecordStoreException ex) {
            ex.printStackTrace();
        } finally {
            SafeCloseRecordStore(s);
        }
    }

    /**
     * parse the servercapabilities from the given string.
     * currently it looks only for the SmartSlowSync ext.
     */
    private static void parseCapabilities(String string) {

        //TODO: complete the implementation looping all the capabilities and exts

        Log.info("[ServerCapFiller.parseCapabilities] ready to parse capabilities: " + string);

        // Currently we look into the ext for the SmartSlowSync field
        char[] sep = {(char) 0x0A};

        String[] lines = StringUtil.split(string, new String(sep));
        String exts = "";
        for (int i = 0; i < lines.length; i++) {

            // looking for the line with exts
            if (lines[i].indexOf(EXT) == 0) {
                //we've found the exts so we can save the line and quit the loop
                exts = lines[i];
                break;
            }
        }

        Vector extVector = new Vector(1);

        // do we have smart slow sync in the exts?
        if (exts.indexOf(Ext.X_FUNAMBOL_SMARTSLOW) != -1) {
            Ext slow = new Ext(Ext.X_FUNAMBOL_SMARTSLOW, new String[0]);
            extVector.addElement(slow);
            Log.info("[ServerCapFiller ParseCapabilities] server supports slow sync!");
        }

        // we can set the property accordingly
        serverCapabilities = new DevInf();
        serverCapabilities.setExts(extVector);


    }

    private static void saveCapabilities() {
        Log.info("[ServerCapFiller] saving config.");
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream os = new DataOutputStream(baos);
        AbstractRecordStore s = null;
        try {
            serialize(os);

            try {

                deleteStoredConfig();
                initRecordStore();
                s = AbstractRecordStore.openRecordStore(STORE_NAME, false);


                // now we have a recordstore opened with 1 records
                s.addRecord(baos.toByteArray(), 0, baos.toByteArray().length);

            } catch (RecordStoreException ex) {
                ex.printStackTrace();
            } finally {
                // close the recordstore
                SafeCloseRecordStore(s);
            }

        } catch (IOException e) {
            // error in serialize
            e.printStackTrace();
        }
    }

    /**
     * get server cap from rms.
     * @return true if server cap has been found, false otherwise
     */
    private static boolean fillServerCapFromStorage() {
        AbstractRecordStore rs = null;
        try {

            Log.info("[ServerCapFiller] getting server cap from storage");
            serverCapabilities = null;

            rs = AbstractRecordStore.openRecordStore(STORE_NAME, false);

            // the recordstore contains version number and the record with the 
            // server capabilities. if we have a number of records that is
            // different from 2 we have an issue
            if (rs.getNumRecords() == 2) {
                Log.debug("[ServerCapFiller] 2 records found, ok. :D");

                RecordEnumeration e = rs.enumerateRecords(null, null, false);
                int id = e.nextRecordId();
                // record with record id = 1 contains the version number. we can
                // throw it away for now.  when we'll have more than one store
                // version we'll   have to handle this correctly
                if (id == 1) {
                    // get the other one!
                    id = e.nextRecordId();
                }

                byte[] data = rs.getRecord(id);

                DataInputStream bais = new DataInputStream(new ByteArrayInputStream(data));

                //deserialize method fills serverCapabilities if able
                deserialize(bais);
            } else {
                Log.info("[ServerCapFiller] " + rs.getNumRecords() + " records found. uh?");
            }

            rs.closeRecordStore();


        } catch (IOException ex) {
            Log.error("[ServerCapFiller] error deserializing rs");
            ex.printStackTrace();

        } catch (RecordStoreNotFoundException ex) {
            Log.info("[ServerCapFiller] server cap recordstore not found.");

        } catch (RecordStoreFullException ex) {
            ex.printStackTrace();
        } catch (RecordStoreException ex) {
            ex.printStackTrace();
        } finally {
            SafeCloseRecordStore(rs);
        }

        if (serverCapabilities != null) {
            Log.info("[ServerCapFiller] server cap loaded from storage");
            return true;
        } else {
            return false;
        }
    }

    /**
     * 
     * @return server capabilities url, or null in case of a config error.
     * @throws ConfigException if config has not been set or is not accessible
     */
    private static String getServerCapURL() throws ConfigException {

        if (serverCapURL == null) {
            MailClientConfig config = ConfigManager.getConfig();

            if (config.getMailAccount().isSet()) {
                serverCapURL =
                        StringUtil.trim(config.getMailAccount().getUrl(), '/') + SERVER_CAP_SUFFIX;
                Log.info("[ServerCapFiller] server cap url is: " + serverCapURL);
            } else {
                throw new ConfigException("Mail Account is not set");
            }

        }
        return serverCapURL;

    }

    private static void serialize(DataOutputStream out) throws IOException {

        Log.info("[ServerCapFiller] Serializing stuff");

        boolean support = false;

        Vector v = serverCapabilities.getExts();

        for (int i = 0; i <
                v.size(); i++) {

            Ext e = (Ext) v.elementAt(i);
            if (e.getXNam().equals(Ext.X_FUNAMBOL_SMARTSLOW)) {
                support = true;
                break;
            }

        }

        out.writeBoolean(support);
        Log.info("servercapfiller writing config: smartslow set to " + support);

    }

    private static void deserialize(DataInputStream in) throws IOException {
        boolean supported = in.readBoolean();
        Log.info("[ServerCapFiller] server supports slow sync: " + supported);
        if (supported) {
            Vector extVector = new Vector(1);
            Ext slow = new Ext(Ext.X_FUNAMBOL_SMARTSLOW, new String[0]);
            extVector.addElement(slow);
            serverCapabilities =
                    new DevInf();
            serverCapabilities.setExts(extVector);
        }
    }
}
