/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
package com.funambol.mailclient.ui.controller;

import com.funambol.mailclient.loc.LocalizedMessages;
import com.funambol.mailclient.ui.view.ModalPopup;
import com.funambol.mailclient.ui.view.SaveConfigPopupAction;
import com.funambol.util.BasicConnectionListener;
import com.funambol.util.Log;

/**
 * This class is an extension of com.funambol.util.BasicConnectionListener. It
 * Overrides the base class methods in order to give the user the possibility to
 * interact with the system's connection processes. For further documentation,
 * please refer to the com.funambol.util.BasicConnectionListener class and 
 * to the com.funambol.util.ConnectionListener interface javadoc. This class 
 * implementation is optimized for Blackberry devices
 */
public class UIConnectionListener extends BasicConnectionListener {

    /**
     * Open up an alert to ask the user if the configuration proposed by the 
     * system could be used or not. This call locks the calling thread before 
     * starting the connection to the network.
     * @param config is the String formatted configuration
     * @return boolean user permission true if the user selects "continue", 
     * false otherwise. 
     */
    public boolean isConnectionConfigurationAllowed(String config) {

        Log.debug("[UIConnectionListener] asking the user if it's ok to use config using popupaction");

        SaveConfigPopupAction popupAction = new SaveConfigPopupAction();

        String title = LocalizedMessages.CAN_USE_CONNECTION_TITLE;
        String text = LocalizedMessages.CAN_USE_CONNECTION1 + " " + config + " " + LocalizedMessages.CAN_USE_CONNECTION2;


        ModalPopup popup = new ModalPopup(title, text, popupAction, LocalizedMessages.POPUP_ACCEPT, LocalizedMessages.POPUP_DENY);
        UIController.showModalPopup(popup, UIController.display.getCurrent());
        Log.debug("[UIConnectionListener] entering sync. code");

        synchronized (SaveConfigPopupAction.lock) {
            try {
                Log.debug("[UIConnectionListener] waiting");
                SaveConfigPopupAction.lock.wait();
                if (SaveConfigPopupAction.response != SaveConfigPopupAction.CONFIRM) {
                    //UIController.display.setCurrent(UIController.backStack.pop());
                    //UIController.showBackScreen();
                    UIController.showInboxScreen();
                    Log.debug("[SaveConfigPopupAction] User denied permission, returning false");
                    return false;
                } else {
                    Log.debug("[SaveConfigPopupAction] User granted permission, returning true");
                    //UIController.showBackScreen();
                    UIController.showInboxScreen();
                    return true;
                   
                }
            } catch (InterruptedException ex) {
                ex.printStackTrace();
                Log.debug("[UIConnectionListener] interrupted");
            }
        }
        //FunambolAlert a = new FunambolAlert(LocalizedMessages.CONNECTION_REFUSED, LocalizedMessages.GO_TO_OPTIONS_AND_ENTER_APN_INFORMATIONS, AlertType.INFO);
        Log.debug("[UIConnectionListener] returning false, but should not be here");
        return false;
    }

    public void connectionConfigurationChanged() {
        //This method is just empty. It should restart the CTP service on the 
        //new found connection configuration. As experimentally proven when a 
        //connection configuration is not valid anymore, the CTPService class
        //can resolve the connection without restarting the service. Anyway, if 
        //in next client release the product will need to use WIFI as first 
        //priority network, the CTP service will need a restart. 
        //This said, uncomment the following lines of code if there are problems 
        //with the usage of the same connection configuration for all of the 
        //client's connection. Here only CTPService is included, but ideally 
        //this method should take care of every classes that need something when 
        //the connection configuration changes in order them to use one only 
        //connection configuration.
        /*try {
            //Notify here all classes that a new connection is required in order to
            //use all the same APN.
            if (ConfigManager.getConfig().getCtpPushStatus() == MailClientConfig.CTP_ENABLED && CTPManager.getInstance().isRunning()) {
                CTPManager.getInstance().restartCTP();
            }
        } catch (ConfigException ex) {
            ex.printStackTrace();
        }*/
    }
}
