/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */
package com.funambol.mailclient.ui.view;

import com.funambol.mail.Address;
import com.funambol.mail.MailException;
import com.funambol.mailclient.cm.AlreadyExistentContactException;
import com.funambol.mailclient.cm.ContactManagerException;
import com.funambol.mailclient.ui.controller.UIController;
//import com.funambol.mailclient.ui.view.FunAddressList;
import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.cm.ContactManager;

import com.funambol.mailclient.loc.Localization;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Choice;
import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.StringItem;
import javax.microedition.lcdui.TextField;

public class AddContactForm extends Form implements CommandListener {

    // mode used to add a contact
    public static final int MODE_ADD_CONTACT = 0;
    // mode used to add a recipient, saving contact is optional
    public static final int MODE_ADD_RECIPIENT = 1;
    // mode used to edit an existing contact
    public static final int MODE_EDIT_CONTACT = 2;
    private int mode = MODE_ADD_CONTACT;
    //TODO: change this according to contact fields
    private static final int MAX_TXTFIELD_SIZE = 128;
    private Command saveCommand = UIController.saveCommand;
    private Command okCommand =
            new Command(Localization.getMessages().GENERIC_OK,
            UIController.COMMAND_TYPE, 1);
    private Command advancedCommand =
            new Command(Localization.getMessages().ACF_ADVANCED_COMMAND_LABEL,
            UIController.COMMAND_TYPE, 5);
    private TextField txtVisibleName = new TextField(Localization.getMessages().ACF_VISIBLE_NAME_LABEL,
            "", MAX_TXTFIELD_SIZE, TextField.ANY);
    private TextField txtEmail1 = new TextField(Localization.getMessages().ACF_CONTACT_EMAIL_1_LABEL,
            "", MAX_TXTFIELD_SIZE, TextField.ANY | TextField.NON_PREDICTIVE);
    private TextField txtFirstName =
            new TextField(Localization.getMessages().ACF_CONTACT_FIRSTNAME_LABEL, "", MAX_TXTFIELD_SIZE, TextField.ANY);
    private TextField txtLastName =
            new TextField(Localization.getMessages().ACF_CONTACT_LASTNAME_LABEL, "", MAX_TXTFIELD_SIZE, TextField.ANY);
    private ChoiceGroup chkSaveContact =
            new ChoiceGroup(null, Choice.MULTIPLE);
    private Contact contact;
    // private boolean isNew;
    private ContactList contactList;
    private Contact existentContact = null;

    /**
     * Creates a new instance of AddContactForm
     */
    public AddContactForm(ContactList contactList) {
        this(new Contact(), MODE_ADD_RECIPIENT);
        this.contactList = contactList;
    //setTitle(Localization.getMessages().ACF_NEW_CONTACT_FORM_TITLE);
    }

    public AddContactForm(Contact contact, ContactList contactList, int mode) {
        this(contact, mode);
        this.contactList = contactList;
    }

    public AddContactForm(Contact contact, int mode) {


        super(getTitleFromMode(mode));
        this.mode = mode;
        this.contact = contact;
        if (mode == MODE_EDIT_CONTACT) {
            this.addCommand(advancedCommand);
            this.addCommand(saveCommand);
        } else {
            this.addCommand(okCommand);
        }




        this.addCommand(UIController.cancelCommand);
        this.setCommandListener(this);
        //#ifdef isBlackberry
        //# this.addCommand( UIController.blackberryExitCommand );
        //#endif
        txtEmail1.setInitialInputMode("MIDP_LOWERCASE_LATIN");
        txtVisibleName.setString(contact.getVisibleName());
        txtEmail1.setString(contact.getDefaultEmail());
        if (!StringUtil.isNullOrEmpty(contact.getFirstName())) {
            txtFirstName.setString(contact.getFirstName());
        }
        if (!StringUtil.isNullOrEmpty(contact.getLastName())) {
            txtLastName.setString(contact.getLastName());
        }

        if (isNameToGuess(contact)) {

            String[] names = getFirstAndLastNames(contact);

            txtFirstName.setString(names[0]);
            txtLastName.setString(names[1]);
        }

        chkSaveContact.append(Localization.getMessages().ACF_SAVE_CONTACT_CHECKBOX, null);
        chkSaveContact.setSelectedIndex(0, true);
        if (mode == MODE_EDIT_CONTACT) {
            append(txtFirstName);
            append(txtLastName);
            append(txtEmail1);
            append(txtVisibleName);
        } else {
            append(txtEmail1);
            append(txtFirstName);
            append(txtLastName);
        }
        if (mode == MODE_ADD_RECIPIENT) {
            append(chkSaveContact);

        }
    }

    /**
     * this method implement an euristic algorithm that tries to guess first
     * and last name from a given contact with only visiblename
     * @param contact a contact with non null visible name and empty first and
     * last name
     * @return an array of size 2 composed by first name and last name
     */
    private String[] getFirstAndLastNames(Contact contact) {

        String[] names = StringUtil.split(contact.getVisibleName(), ",");
        String fname = null;
        String lname = null;

        if (names.length == 2) {
            // we're in the case of "Doe, John" so we 
            // can guess that first name is the second part
            fname = names[1];
            lname = names[0];
        } else if (names.length > 2) {
            // we're in the case of "Huey, Dewey, Louie" 
            // so we use it as first name and live happy everafter
            fname = contact.getVisibleName();
        } else {
            // splitting with spaces
            names = StringUtil.split(contact.getVisibleName(), " ");

            // we're in the case of "John Doe", we can suppose that 
            // the last word is the last name, and the remaining is 
            // the first name
            lname = names[names.length - 1];
            if (names.length > 1) {
                // putting what remains in fname
                fname = contact.getVisibleName().substring(0, contact.getVisibleName().length() - lname.length() - 1);
            }
        }
        return new String[]{fname, lname};
    }

    public void commandAction(Command command, Displayable displayable) {
        if (command == okCommand || command == saveCommand) {
            try {

                // avoid invalid email addresses
                if (txtEmail1.getString().indexOf('@') <= 0) {
                    UIController.showErrorAlert(Localization.getMessages().INVALID_EMAIL_ADDRESS);
                    return;
                }

                // build the contact 
                contact.setDefaultEmail(txtEmail1.getString());
                contact.setNickName(txtVisibleName.getString().replace('\n', ' ').trim());
                contact.setFirstName(txtFirstName.getString().replace('\n', ' ').trim());
                contact.setLastName(txtLastName.getString().replace('\n', ' ').trim());


                switch (mode) {
                    case MODE_EDIT_CONTACT:

                        Log.debug("Mode = EDIT_CONTACT, saving existing contact");
                        // we want to save.
                        UIController.saveContact(contact, false);
                        if (contactList != null) {
                            contactList.resetActiveElementHeight();
                        }
                        break;
                    case MODE_ADD_CONTACT:
                        Log.debug("Mode = ADD_CONTACT, saving new contact");
                        saveContact();
                        break;

                    case MODE_ADD_RECIPIENT:
                        Log.debug("Mode = ADD_RECIPIENT");
                        if (chkSaveContact.isSelected(0)) {
                            Log.debug(" checkbox selected, saving contact");
                            saveContact();
                        } else {
                            // adding the recipient to the list
                            Log.debug("adding address to message");
                            contactList.addAddress(contact.getAddress(Address.TO));
                        }
                        break;
                    default:
                        break;
                }
                UIController.showBackScreen();

            } catch (ContactListFullException ex) {
                  UIController.showErrorAlert(
                        Localization.getMessages().AB_LIMIT_ERROR_MSG);
            } catch (ContactManagerException ex) {
                Log.error("contact manager error when saving contact: " + ex.toString());
                ex.printStackTrace();
            }

        } else if (command == advancedCommand) {
            UIController.showAdvancedContactScreen(contact);
        } else if (command == UIController.cancelCommand) {
            UIController.showBackScreen();
        } else if (command == UIController.blackberryExitCommand) {
            UIController.midlet.destroyApp(false);
        }
    }

    private void saveContact() throws ContactListFullException {
        {
            if (UIController.checkContactListFull()) {
                Log.info("Contact list is full");
                throw new ContactListFullException();
              
            } else {
             
                if (txtEmail1.getString().indexOf('@') <= 0) {
                    UIController.showErrorAlert(
                            Localization.getMessages().INVALID_EMAIL_ADDRESS);
                } else {
                    try {
                        //  if ((isValidContact())) {
                        disableCommands();
                        setTitle(Localization.getMessages().SAVING_CONTACT);
                        UIController.getThreadPool().startThread(
                                new Thread() {

                                    public void run() {
                                        try {

                                            if (contact.getContactId() == 0) {
                                                // the flag new can be true also for contact
                                                // not in rms, e.g. if they came from a
                                                // reply message.
                                                UIController.saveContact(contact, true);
                                            } else {
                                                UIController.saveContact(contact, false);
                                            }

                                            if (contactList != null) {
                                                if (mode == MODE_ADD_RECIPIENT) {
                                                    contactList.addAddress(contact.getAddress(Address.TO));
                                                    contactList.SortByState();
                                                } else {
                                                    // we can be in edit, and need to recalculate
                                                    // the height of the element
                                                    contactList.resetActiveElementHeight();
                                                }
                                            }
                                        } catch (AlreadyExistentContactException ex) {
                                            try {
                                                existentContact = UIController.getContactManager().exists(txtEmail1.getString());
                                                contactList.activateElement(existentContact);
                                                UIController.backStack.push(UIController.display.getCurrent());
                                                UIController.showAlert(
                                                        Localization.getMessages().CONTACT_ALREADY_EXISTS_EDITING_IT,
                                                        UIController.showEditContactScreen(
                                                        //null,ex.getExistentContact(),contactList, false)
                                                        null, existentContact, contactList, MODE_EDIT_CONTACT));
                                            } catch (ContactManagerException e) {
                                                ;
                                            }


                                        } catch (ContactManagerException ex) {
                                            Log.error(this, "Error creating contact: " + ex.getMessage());
                                            if (UIController.checkContactListFull()) {
                                                UIController.showErrorAlert(
                                                        Localization.getMessages().AB_LIMIT_ERROR_MSG);
                                            } else {
                                                UIController.showErrorAlert(Localization.getMessages().INVALID_EMAIL_ADDRESS);
                                                ex.printStackTrace();
                                            }
                                        }

                                    }
                                });

                    } catch (MailException ex) {
                        Log.error(this, "Error adding contact to AddressList");
                        UIController.showErrorAlert(Localization.getMessages().ACF_ALERT_ERROR_SAVING_CONTACT_MESSAGE);
                        // contactList.reinitializeContactItems();
                        ex.printStackTrace();
                    }
                }
            }

        }
    }

    private void disableCommands() {
        this.removeCommand(saveCommand);
        this.removeCommand(advancedCommand);
        this.removeCommand(UIController.cancelCommand);
    }

    private boolean isValidContact() {

        try {
            existentContact = UIController.getContactManager().
                    exists(txtEmail1.getString());
        } catch (Exception ex) {
            ex.printStackTrace();
            Log.error("[AddContactForm] Error validating contact" + ex.getMessage());
        }

        if (mode != MODE_EDIT_CONTACT && existentContact == null) {
            return true;
        } else if (mode != MODE_EDIT_CONTACT && existentContact != null) {
            return false;
        } else if (mode != MODE_EDIT_CONTACT && contact.getDefaultEmail().equals(txtEmail1.getString())) {
            return true;
        } else if (mode != MODE_EDIT_CONTACT && existentContact == null) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * method to find out if we need to guess first name and last name for the
     * given contact
     * @param contact
     * @return true if we should guess the name (i.e. 
     * if visible name is not empty but first and last name are empty)
     * 
     */
    private boolean isNameToGuess(Contact contact) {
        //return true if we have visible name but no first and last name
        return (!StringUtil.isNullOrEmpty(contact.getVisibleName()) &&
                StringUtil.isNullOrEmpty(contact.getFirstName()) &&
                StringUtil.isNullOrEmpty(contact.getLastName()));
    }

    private static String getTitleFromMode(int mode) {

        switch (mode) {
            case MODE_ADD_RECIPIENT:
                // add recipient
                Log.debug("mode = add recipient:  returning " + Localization.getMessages().ACF_ADD_RECIPIENT_FORM_TITLE);
                return Localization.getMessages().ACF_ADD_RECIPIENT_FORM_TITLE;

            case MODE_EDIT_CONTACT:
                // edit
                Log.debug("mode = edit contact: returning " + Localization.getMessages().ACF_EDIT_CONTACT_FORM_TITLE);
                return Localization.getMessages().ACF_EDIT_CONTACT_FORM_TITLE;

            default:
                // add contact
                Log.debug("mode = default: returning " + Localization.getMessages().ACF_NEW_CONTACT_FORM_TITLE);
                return Localization.getMessages().ACF_NEW_CONTACT_FORM_TITLE;
        }

    }
}
