/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


package com.funambol.mailclient.ui.view;

import com.funambol.util.StringUtil;
import java.util.Vector;

/**
 * Basic implementation of ContactItemFilter.
 * Used to select contacts with visible name, first name or last name starting
 * with given string (case insensitive)
 */
public class BasicContactItemFilter  implements ContactItemFilter {
    
    private String searchString;
    
    /**
     * Creates a new instance of BasicContactItemFilter setting
     * the filter string to the given one
     *
     * @param s the string to look for
     */
    public BasicContactItemFilter(String s) {
        this.searchString = s;
    }
    
    /**
     * Creates a new instance of BasicContactItemFilter setting
     * the filter string to null
     */
    public BasicContactItemFilter() {
        this(null);
    }
    
    
    /**
     * Check if the given contactItem matches the current search string
     * @return true if current string is null or if Visible name, first name
     * or last name of the contactitem starts with the current string
     */
    public boolean match(ContactItem c) {
        if (searchString == null) {
            return true;
        }
        // selected addresses must always be shown
        if(c.getState() != ContactItem.ADDRESS_STATE_NONE) {
            return true;
        }
        
        return ( smartStartsWith( c.getContact().getVisibleName(),searchString ) ||
                smartStartsWith( c.getContact().getFirstName(), searchString ) ||
                smartStartsWith( c.getContact().getLastName(), searchString ));
        
    }
    
    /**
     * return true if tocheck starts with start (case insensitive)
     */
    private boolean smartStartsWith(String toCheck, String start) {
        
        if (StringUtil.isNullOrEmpty(toCheck)) {
            return false;
        }
        return toCheck.toLowerCase().startsWith(start.toLowerCase());
    }
    
    /**
     * @return the current search string
     */
    public String getSearchString() {
        return searchString;
    }
    
    /**
     * set the search string
     * @param newString the new search string. can also be null.
     */
    public void setSearchString(String newString) {
        if (searchString != newString) {
            this.searchString = newString;
        }
    }
    
    
    
    /**
     * filter the contactitems
     * @param contactitems the fill contact items list
     * @filteredVector unused
     */
    public Vector filter(Vector contactItems, Vector filteredVector)
         throws NoMatchingContactsException{
        
        Vector ret = new Vector(contactItems.size());
        
        for (int i =0; i< contactItems.size(); i++) {
            if( match( (ContactItem)contactItems.elementAt(i)) ) {
                ret.addElement(contactItems.elementAt(i));
            }
        }
        
        ret.trimToSize();
        
        if (ret.size() == 0) {
            throw new NoMatchingContactsException();
        }
        return ret;
        
    }
    
    /**
     *set the search string to null.
     * this cause subsequent calls to match to return true for every contactitem
     * until a new search string has been set using setSearchString method
     */
    public void clear() {
        searchString = null;
    }
    
    
    
}