/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 */
package com.funambol.mailclient.ui.view;

import com.funambol.mail.Address;
import com.funambol.mail.MailException;
import com.funambol.mail.Message;
import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.loc.LocalizedMessages;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.mailclient.loc.Localization;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import com.funambol.util.ThreadPool;
import java.util.Enumeration;
import java.util.Vector;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

public class ContactList extends List implements
        ContactItemCollectionListener {

    // margin constants
    private static final int V_MARGIN = 3;
    private static final int H_MARGIN = 5;
    // private static Font labelFont;
    private static Font titleFont;
    private Drawer drawer;
    /**
     * the message from wich the recipients are shown and to which the
     * recipients are set. It's protected to allow access from the
     * recipient list
     */
    protected Message message;
    /**
     * the number to be shown in the upper left corner,
     */
    private int contactCount;
    // commands
    private Command syncCommand;
    private Command addContactCommand;
    private Command okCommand;
    private Command selectCommand;
    private Command editContactCommand;
    private Command deleteContactCommand;
    private Command saveContactCommand;
    private Command resetContactsCommand;
    protected Command toContactCommand;
    protected Command ccContactCommand;
    protected Command bccContactCommand;
    protected Command clearContactCommand;
    private Command cancelCommand;
    private ContactItemCollection coll;
    //private Command callCommand;
    private Command searchCommand;
    private Command clearSearchCommand;
    private T9Search t9;
    private BasicContactItemFilter basicFilter;
    // are we filtering the contact list?
    private boolean filtering = false;
    private String[] subtitle;

    public ContactList() {
        this(new Message());

    }

    /** Creates a new instance of ContactList with the given message */
    public ContactList(Message message) {
        super();
        this.message = message;
        this.drawer = UIController.getDrawer();
        this.titleFont = drawer.getGraphicalTheme().getTitleFont();
        setSubtitle(Localization.getMessages().KEYPAD_TO_SEARCH);
        setTitle(getDefaultTitle());
        setCommandListener(this);
        t9 = new T9Search();
        basicFilter = new BasicContactItemFilter();
        initializeContactList();
    }

    /*
     * Used to add/remove menu commands based on the current selection.
     * @see com.funambol.mailclient.ui.view.FunCanvasList#updateItemContext()
     */
    protected void updateItemContext() {
        //Null check
        if (getActiveContactItem() == null) {
            return;
        }

        //Happens when we are viewing contacts in an email. 
        if (clearContactCommand == null || bccContactCommand == null ||
                ccContactCommand == null || toContactCommand == null) {
            return;
        }

//#ifdef isBlackberry
//#         //For BlackBerry
//#         //Start with all item commands. Remove ones we do not want.
//#        addItemCommands();
//#        switch (getActiveContactItem().getState()) {
//#            case ContactItem.ADDRESS_STATE_NONE: {
//#                removeCommand( clearContactCommand );
//#                enableDisableOKCommand();
//#                break;
//#            }
//#            case Address.BCC: {
//#                removeCommand( bccContactCommand );
//#                addCommand( okCommand );
//#                addCommand( clearContactCommand );        
//#                break;
//#            }
//#            case Address.CC: {
//#                removeCommand( ccContactCommand );
//#                addCommand( okCommand );
//#                addCommand( clearContactCommand );
//#                break;
//#            }
//#            case Address.TO: {
//#                removeCommand( toContactCommand );
//#                addCommand( okCommand );
//#                addCommand( clearContactCommand );
//#                break;
//#            }
//#        }
//#endif
    }

    /**
     * reset the contact list using the given message.
     * @param message the message
     */
    public void resetContactList(Message message) {
        this.message = message;
        setContactCount(getRmsCount());
        setTitle(getDefaultTitle());
        resetSubtitle();
        emptySearchString();
        initializeContactList();
    }

    /**
     * @return the default title
     */
    public String getDefaultTitle() {
        return Localization.getMessages().SELECT_RECIPIENTS;
    }

    int paintElement(int elementId, Graphics g) {
        int h = 0;
        //can be null if trying to paint during a sync
        //Log.debug("painting element " + elementId);
        if (elements.elementAt(elementId) != null) {
            g.translate(0, current_paint_h);
            h = ((ContactItem) elements.elementAt(elementId)).paint(g, isActive(elementId));
            g.translate(0, -current_paint_h);
        }
        return h;
    }

    public ContactItem getActiveContactItem() {
        return (ContactItem) getActiveElement();
    }

    private boolean isLeftKey(int dir, int keyCode) {
        boolean isLeftKey =
                ((dir == Canvas.LEFT) &&
                ((keyCode < 0) || (keyCode == Canvas.LEFT)));

        return isLeftKey;
    }

    private boolean isRightKey(int dir, int keyCode) {
        boolean isRightKey =
                ((dir == Canvas.RIGHT) &&
                ((keyCode < 0) || (keyCode == Canvas.RIGHT)));

        return isRightKey;
    }

    void handleKey(int keyCode) {

        int dir = getGameAction(keyCode);
        //Spacebar events do not map with getGameAction. Use keycode
        //#ifdef isBlackberry
        //keycode 32 is spacebar
        //# if(keyCode == 32) {
        //#    if(getActiveContact() != null) {
        //#        commandAction( toContactCommand, this );
        //#    }
        //#    return;
        //keycode 10 is enter
        //# } else if (keyCode == 10) {
        //#    if (getActiveContactItem().hasMultipleEmails()) {
        //#        getActiveContactItem().toNextEmail();
        //#    }
        //# //Keycode 8 is delete on blackberry
        //# } else if (keyCode == 8) {
        //#    if (getActiveContact() !=null) {
        //#        commandAction( deleteContactCommand,this);
        //#    }
        //# }
        //#endif

        if (isLeftKey(dir, keyCode)) {
            //  Log.debug("Left!");
            if (getActiveContactItem().hasMultipleEmails()) {
                getActiveContactItem().toPreviousEmail();
            } else {
                scroll(-1);
            }
            repaint();
            return;
        } else if (isRightKey(dir, keyCode)) {
            //   Log.debug("Right!");
            if (getActiveContactItem().hasMultipleEmails()) {
                getActiveContactItem().toNextEmail();
            } else {
                scroll(1);
            }
            repaint();
            return;
        }
        /*
        Log.debug("keycode= " + keyCode + ", gameAction= " + getGameAction(keyCode) +
        ", keyname= " + getKeyName(keyCode) + " Char: " +  (char) keyCode);
        keyMapper.addKey(keyCode);
        searchString= keyMapper.getStrings();
         */




        // we set the filter if needed

        if (coll.getFilter() == basicFilter) {
            // we need to add the current string to the t9 field
            t9.setString(basicFilter.getSearchString());
            coll.setFilter(t9);
        }
        // clear
        if (UIController.isClearKey(keyCode)) {
            coll.getFilter().clear();
            coll.setToBeFiltered(true);

        //#ifdef searchform
//#             removeCommand(clearSearchCommand);
//#             addCommand(searchCommand);
        //#endif
        } else if ((keyCode > 0) && (!isSoftKey(dir, keyCode))) {
            Log.info("adding key " + keyCode);
            t9.addKey(keyCode);
            coll.setToBeFiltered(true);
        //#ifdef searchform
//#             removeCommand(searchCommand);
//#             addCommand(clearSearchCommand);
        //#endif
        }

        if (coll.isToBeFiltered()) {

            //repainting only the subtitle
            filtering = true;
            setSubtitle(generateSubtitle());
            repaint();
            serviceRepaints();

            try {
                super.setElements(coll.getItems(coll.NONE));
                filtering = false;
                setSubtitle(generateSubtitle());
                repaint();
                rewindList();
            } catch (NoMatchingContactsException ex) {
                // nothing to do if no contacts has been found
                Log.info("No matching contacts found, nothing to change.");
                setSubtitle(Localization.getMessages().NO_MATCHING_RESULTS);
                repaint();
                serviceRepaints();
                //  ex.printStackTrace();
                filtering = false;
            }
        }
    }

    protected void paint(Graphics graphics) {

        if (graphics.getClipHeight() < getHeight()) {

            // cleaning title & subtitle background
            Log.info("cleaning background");
            graphics.setColor(UIController.getDrawer().getGraphicalTheme().
                    getBackgroundColor());
            //graphics.setColor(0xff0000);
            graphics.fillRect(0, 0, graphics.getClipWidth(), graphics.getClipHeight());
            paintTitle(graphics);

        } else {
            super.paint(graphics);
        }
    }

    /**
     * empty the search string
     */
    public void emptySearchString() {
        coll.getFilter().clear();
    }

    protected void initCommands() {
        int callCmdType = Command.BACK;
//#ifdef UseOKCmd
//#         callCmdType = UIController.COMMAND_TYPE;
//#endif

        okCommand =
                new Command(Localization.getMessages().AL_OK_COMMAND,
                UIController.COMMAND_TYPE, 5);

        selectCommand =
                new Command(Localization.getMessages().SELECT_COMMAND,
                UIController.COMMAND_TYPE, 6);
        toContactCommand = new Command(Localization.getMessages().TO_LABEL, UIController.COMMAND_TYPE, 21);
        ccContactCommand = new Command(Localization.getMessages().CC_LABEL, UIController.COMMAND_TYPE, 22);
        bccContactCommand = new Command(Localization.getMessages().BCC_LABEL, UIController.COMMAND_TYPE, 23);
        clearContactCommand = new Command(Localization.getMessages().REMOVE_CONTACT_COMMAND, UIController.COMMAND_TYPE, 24);
        editContactCommand =
                new Command(Localization.getMessages().EDIT_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 30);

        deleteContactCommand =
                new Command(Localization.getMessages().DELETE_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 40);

        searchCommand =
                new Command(Localization.getMessages().AL_SEARCH_COMMAND,
                UIController.COMMAND_TYPE, 50);
        clearSearchCommand =
                new Command(Localization.getMessages().AL_CLEAR_SEARCH_COMMAND,
                UIController.COMMAND_TYPE, 50);
        addContactCommand =
                new Command(Localization.getMessages().AL_ADD_CONTACT_COMMAND,
                UIController.COMMAND_TYPE, 29);
        syncCommand =
                new Command(Localization.getMessages().AL_SYNC_COMMAND,
                UIController.COMMAND_TYPE, 70);
        resetContactsCommand =
                new Command(Localization.getMessages().RESET_CONTACTS_COMMAD,
                UIController.COMMAND_TYPE, 80);
        saveContactCommand =
                new Command(Localization.getMessages().ABF_SAVE_CONTACT_COMMAND,
                UIController.COMMAND_TYPE, 70);
        cancelCommand = new Command(Localization.getMessages().CANCEL_COMMAND, Command.BACK, 90);

    //callCommand =
    //        new Command(Localization.getMessages().CALL_NUMBER_COMMAND,
    //        callCmdType, 1);

    }

    protected void addCommands() {
        super.addCommands();
        addCommand(cancelCommand);

        // Do not show "Ok" until a contact is selected in TO
        // Now "addCommand(okCommand);" is in toNextState() and in addAddress
        // methocs in the ContactItem class
        //addCommand(okCommand);

        // be sure to have not null addContactCommand
        if (addContactCommand != null) {
            addCommand(addContactCommand);
        }

        enableSyncCommand(UIController.isOkToAddOnlineCommands());

    }

    /**
     * called when the contact list is set to a non empty list
     * removes item-specific commands
     */
    void removeItemCommands() {
        removeCommand(selectCommand);
        removeCommand(editContactCommand);
        removeCommand(deleteContactCommand);


        //#ifdef isBlackberry
//#        if (toContactCommand != null) {
//#            removeCommand( toContactCommand );
//#        }
//#        if (ccContactCommand != null) {
//#            removeCommand( ccContactCommand );
//#        }
//#        if (bccContactCommand != null) {
//#            removeCommand( bccContactCommand );
//#        }
//#        if (clearContactCommand != null) {
//#            removeCommand( clearContactCommand );
//#        }
        //#endif

        setFireCommand(null);
    //#ifdef searchform
    //#     removeCommand(searchCommand);
    //#endif
    }

    /**
     * called when the contact list is set to a non-empty list.
     * adds item-specific commands
     */
    protected void addItemCommands() {

        //#ifdef isBlackberry
//#     if (toContactCommand != null) {
//#         addCommand( toContactCommand );
//#         setFireCommand( toContactCommand );
//#     }
//#     if (ccContactCommand != null) {
//#         addCommand( ccContactCommand );
//#     }
//#     if (bccContactCommand != null) {
//#         addCommand( bccContactCommand );
//#     }
//#     if (clearContactCommand != null) {
//#         addCommand( clearContactCommand );
//#     }
        //#else
        addCommand(selectCommand);
        setFireCommand(selectCommand);
        //#endif
        addCommand(editContactCommand);
        addCommand(deleteContactCommand);

    //#ifdef searchform
    //#     addCommand(searchCommand);
    //#endif
    }

    void paintBackground(Graphics graphics) {
        drawer.drawCanvasBackground(graphics, getWidth(), getHeight());
    }

    /**
     * paint the title
     * @return the height of the painted area
     */
    public int paintTitle(Graphics g) {

        //  Log.debug("paint title!");

        String s;
        //String title = getTitle();

        // Use method getFunTitle instead getTitle for
        // double titlebar in Nokia S40
        String title = getFunTitle();

        // keep this because otherwhise some motorola phones may throw a
        // nullpointer exception
        //TODO: check if still true when using canvas!
        if (title == null) {
            //    Log.debug("title null... setting to empty string");
            title = "";
        }
        int num = Math.max(elements.size(), contactCount);
        if (elements.size() > 0) {
            s = " " + (getActiveElementId() + 1) + "/" + num;
        } else if (num == 0) {
            s = " (Empty)";
        } else {
            s = " " + num;
        }
        int sWidh = titleFont.stringWidth(s);

        String shortTitle = UiUtils.cutString(title, getWidth() - sWidh, titleFont);
        drawer.drawTitleBackground(g, getWidth(), titleFont.getHeight());
        g.setFont(titleFont);
        g.setColor(drawer.getGraphicalTheme().getTitleColor());
        //not using drawTitle because we need to draw also the number of messages
        g.drawString(shortTitle, 0, 0, Graphics.TOP | Graphics.LEFT);
        g.drawString(s, getWidth(), 0, Graphics.TOP | Graphics.RIGHT);


        int h = titleFont.getHeight();

        g.translate(0, h);
        int t = drawSubtitle(g);
        g.translate(0, -h);
        return h + t;
    }

    /**
     *  draw a subtitle giving the users hints about how to use the keypad to
     *  search contacts
     *  @param g the Graphincs object to paint on
     *  @param h the height of the already painted area. drawSubtitle should not
     *  paint at
     *  @return the height of the painted area
     */
    protected int drawSubtitle(final Graphics g) {
        //not drawing if no contacts are present
        if (elements.size() == 0) {
            return 0;
        }
        g.setColor(drawer.getGraphicalTheme().getForegroundColor());
        for (int i = 0; i < subtitle.length; i++) {
            g.drawString(subtitle[i], getWidth() / 2,
                    drawer.getGraphicalTheme().getTitleFont().getHeight() * i,
                    Graphics.TOP | Graphics.HCENTER);
        }
        int h2 = drawer.getGraphicalTheme().getTitleFont().getHeight() * (subtitle.length);

        g.drawLine(0, h2, getWidth(), h2);
        return h2;
    }

    /**
     * @return true if element is activable.
     */
    boolean isActivable(int elementId) {
        return isVisible(elementId);
    }

    /**
     * @return true if element must be shown
     */
    boolean isVisible(int elementId) {
        return true;
    }

    public void commandAction(Command command, Displayable displayable) {
        if (command == cancelCommand) {

            //if this is not null, we reset the title
            if (UIController.composeMessageScreen != null) {
                UIController.getComposeMessageScreen().setDefaultTitle();
            }
            // we may have some contactitems in the middle of the list
            // to a state other than NONE, we need to clear them!
            int num = UiUtils.getMessageAddresses(message).size();
            for (int i = num; i < elements.size(); i++) {
                ContactItem c = (ContactItem) elements.elementAt(i);
                c.setState(ContactItem.ADDRESS_STATE_NONE);
            }
            
            
            if (! (UIController.backStack.peek() instanceof ComposeMessageForm)) {
            removeNotStoredContacts();
            }
            UIController.showBackScreen();
        } // if we have the search command
        else if (command == searchCommand) {
            UIController.display.setCurrent(new SearchAddressForm(this));
        } else if (command == clearSearchCommand) {
            // sending a 0 to clear the search
            this.handleKey(Canvas.KEY_NUM0);
        } else if (command == okCommand) {

            // we may have some contacts in the current message that have
            // been unselected, we need to put them back in the right position

            coll.partialSort(0, UiUtils.getMessageAddresses(message).size(), coll.ALPHABETICALLY);


            updateMessageRecipients();
            UIController.updateComposeMessageScreen(message);
            /*
            ContactItemComparator c = getContactItemCollection().getComparator();
            getContactItemCollection().setComparator(new StateContactItemComparator());
            getContactItemCollection().forceSort();
            getContactItemCollection().setComparator(c);
            getContactItemCollection().setToBeSorted(false);
             */
            UIController.showComposeMessageScreen(
                    null, UIController.getComposeMessageScreen().subject);

        /*
        UIController.getThreadPool().startThread(
        new Thread() {
        public void run() {
        Log.debug("Setting coll message");
        coll.setMessage(message);
        if (coll.isToBeSorted()) {
        //Log.debug("Thread sorting contacts");
        coll.forceSort();
        //Log.debug("finished sorting contacts");
        }
        }
        }
        );*/



        } else if (command == addContactCommand) {
            updateMessageRecipients();
            UIController.showAddContactScreen(UIController.display.getCurrent(), this);
        } else if (command == syncCommand) {
            this.setTitle(Localization.getMessages().FAL_SYNC_IN_PROGRESS);
            UIController.setSyncCaller(UIController.USER);
            UIController.synchContacts();
        } //item command
        else if (command == selectCommand) {
            if (getActiveContactItem().toNextState() != ContactItem.ADDRESS_STATE_NONE) {
                //we can add the ok command
                addCommand(okCommand);
            } else {
                enableDisableOKCommand();
            }
            ;
            repaint();
        } else if (command == toContactCommand) {
            getActiveContactItem().setState(Address.TO);
            updateItemContext();
            repaint();
        } else if (command == ccContactCommand) {
            getActiveContactItem().setState(Address.CC);
            updateItemContext();
            repaint();
        } else if (command == bccContactCommand) {
            getActiveContactItem().setState(Address.BCC);
            updateItemContext();
            repaint();
        } else if (command == clearContactCommand) {
            getActiveContactItem().setState(ContactItem.ADDRESS_STATE_NONE);
            updateItemContext();
            repaint();
        } else if (command == editContactCommand) {
            UIController.showEditContactScreen(this, getActiveContact(), this, AddContactForm.MODE_EDIT_CONTACT);
        } else if (command == deleteContactCommand) {
            deleteContact();
        //   removeContactItem(getActiveElementId());
        //setContactCount(getRmsCount());
        } else if (command == resetContactsCommand) {

            ModalPopup popup =
                    new ModalPopup(Localization.getMessages().RESET_REQUEST,
                    Localization.getMessages().RESETCONTACTS_MESSAGE,
                    new ResetContactsPopupAction());
            UIController.showModalPopup(popup, this);
        } //else if (command == callCommand){
        //     UIController.phoneCall(getActiveContact(), this);
        // }
        else {
            super.commandAction(command, displayable);
        }
    }

    private void deleteContact() {
        Contact contact = getActiveContact();
        if (UIController.getConfig().getDeleteConfirmation()) {
            ModalPopup popup =
                    new ModalPopup(Localization.getMessages().POPUP_DELETE_CONTACT_TITLE,
                    Localization.getMessages().POPUP_DELETE_CONTACT_TEXT,
                    new DeleteContactPopupAction(contact));
            UIController.showModalPopup(popup, this);
        } else {
            UIController.deleteContact(contact);
            // The ContactManager will inform all listeners (including this
            // object) that an item has been deleted. But the update
            // implementation does not force a repaint, so we must do it here.
            repaint();
        }
    }

    private void updateMessageRecipients() {
        message.removeAllRecipients(Address.TO);
        message.removeAllRecipients(Address.CC);
        message.removeAllRecipients(Address.BCC);

        Address address;
        Vector selected = coll.getSelectedItems();
        for (int i = 0; i < selected.size(); i++) {
            ContactItem item = (ContactItem) selected.elementAt(i);
            address = item.getAddress();
            if (address != null) {
                //    Log.debug("Address: " + address.getEmail());
                try {
                    message.addRecipient(address);

                } catch (MailException me) {
                    Log.error(this, "mail exception updateMessageRecipients:" + me.toString());
                    me.printStackTrace();
                }
            }
        }
    }

    /**
     * initialize the contact list
     */
    public void initializeContactList() {
        initializeContactList(ContactItemCollection.ALPHABETICALLY);
    }

    protected void setContactItemList(final Vector contactItems) {
        // Log.debug("Setting new contact item vector of " + contactItems.size() + " elements");
        super.setElements(contactItems);
        enableDisableOKCommand();
        updateItemContext();
        repaint();
    }

    public Contact getActiveContact() {
        if (getActiveContactItem()!=null) {
            return getActiveContactItem().getContact();
        } else {
            return null;
        }
    }

    /**
     * set the search string array to the given array.
     *
     * @param string an array containing the strings to be used by
     * isVisible function ( if the contact first name, last name or visiblename
     * starts for one of the given strings, the element is visible )
     */
    public void setSearchString(String string) {
        ContactItemFilter oldFilter = coll.getFilter();
        String oldString = null;
        if (oldFilter == basicFilter) {
            // keeping the old string in case we need a rollback
            oldString = basicFilter.getSearchString();
        }

        coll.setFilter(basicFilter);
        basicFilter.setSearchString(string);
        coll.setToBeFiltered(true);
//#ifdef searchform
//#        removeCommand(searchCommand);
//#        addCommand(clearSearchCommand);
//#endif
        try {
            this.setElements(coll.getItems(coll.NONE));
            setSubtitle(Localization.getMessages().ZERO_TO_CLEAR);
        } catch (NoMatchingContactsException ex) {
            Log.info("no matching contacts found, doing nothing");

            this.setSubtitle(Localization.getMessages().NO_MATCHING_RESULTS);
            //rollback
            coll.setFilter(oldFilter);
//#ifdef searchform
//#            removeCommand(clearSearchCommand);
//#            addCommand(searchCommand);
//#endif
            if (oldString != null) {
                // if we're here we have set the filter back to basicfilter, so
                // we can set the string back to what it was
                basicFilter.setSearchString(oldString);
            }
        }
        rewindList();
        repaint();

    }

    /**
     * add an address to the list of the message recipients
     * @param address the adderss to be added
     */
    public void addAddress(Address address) {
        ContactItemComparator oldComparator = coll.getComparator();
        try {
            // apart from this new address, the contact
            // list is sorted, so we don't need to use the complete sorter
            // but we can just use the one based on the state of the items
            message.addRecipient(address);
            coll.addAddress(address);
            addCommand(okCommand);
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error("Unable to add recipient " + address.getEmail() + "to message");
        } finally {
            SortByState();
        }

    }

    /**
     * calls resetHeight() method of the active contact item.
     * This need to be called when the active elements height should be
     * recalculated, maybe because a secondary email address has been added
     */
    void resetActiveElementHeight() {
        getActiveContactItem().resetHeight();
    }

// check if we should add or remove the ok command.
// ok command is added if there is at least one
    protected void enableDisableOKCommand() {
        if (elements.size() == 0) {
            removeCommand(okCommand);
            return;
        }

        // check current selected item before, since it has higher probability
        // of being a recipient
        if (getActiveContactItem().getState() != ContactItem.ADDRESS_STATE_NONE) {
            addCommand(okCommand);
            return;
        }


        // check each item
        for (int i = 0; i < elements.size(); i++) {
            if (((ContactItem) elements.elementAt(i)).getState() != ContactItem.ADDRESS_STATE_NONE) {
                addCommand(okCommand);
                break;

            } else {
                // no contact is in TO, CC or BCC. booo!
                removeCommand(okCommand);
            }

        }
    }

// return contact count
    private int getRmsCount() {
        return UIController.getContactManager().getContactCount();
    }

    /**
     * set the contact count to be shown in the upper left corner
     * @param count the contact count to be shown in the upper left corner,
     * if a negative number is passed, the contact count used will be the
     * total number of contacts in rms
     */
    public void setContactCount(int count) {
        if (count < 0) {
            count = getRmsCount();
        }
        contactCount = count;

    }

    /**
     * enable or disable the sync command and the reset contacts command
     * @param enable if true enable the command, if false disable the command
     */
    public void enableSyncCommand(boolean enable) {
        Log.debug(this, "Enable sync commands: " + enable);
        if (enable) {
            //These are null when viewing an email's contacts. Perform null check. 
            if (syncCommand != null) {
                addCommand(syncCommand);
            }
            if (resetContactsCommand != null) {
                addCommand(resetContactsCommand);
            }
        } else {
            removeCommand(syncCommand);
            removeCommand(resetContactsCommand);
        }
    }

    /**
     * if the given contact is in the list, it is set as the active element.
     * @param contact the contact that must be set as active element
     *
     */
    public void activateElement(Contact contact) {
        int id = -1;
        for (int i = 0; i < elements.size(); i++) {
            if (((ContactItem) elements.elementAt(i)).getContact().getDefaultEmail().equals(contact.getDefaultEmail())) {
                id = i;// contactItems.elementAt(i).getElementID();

                break;
            }
        }
        if (id > 0) {
            setActiveElement(id);
        }

    }

    /**
     * Scan the contact list looking for a contact with given
     * email addres as the default email
     * @param email is the given contact mail
     * @return the Contact with the give email address , or null if
     * the contact is not present
     */
    public Contact exists(String email) {
        for (int i = 0; i < elements.size(); i++) {
            if (((ContactItem) elements.elementAt(i)).getContact().getDefaultEmail().equals(email)) {
                return ((ContactItem) elements.elementAt(i)).getContact();
            }
        }
        return null;
    }

    /**
     * remove the specified contat item from the
     * contactitem vector.
     * @param toRemove the position of the item
     * to be removed in the elemnts vector
     */
    private void removeContactItem(int toRemove) {

        elements.removeElementAt(toRemove);
        if (elements.size() < 1) {
            removeItemCommands();
        }
    }

// is given element active?
    private boolean isActive(int elementId) {
        return elementId == getActiveElementId();
    }

//return contact item collection
    private ContactItemCollection getContactItemCollection() {
        if (coll == null) {
            //  Log.debug("Creating contactitemCollection for the very first time");
            coll = new ContactItemCollection(message);
            coll.setListener(this);
            coll.setFilter(t9);
        }
        return coll;
    }

    /**
     * set the contact list message. contact list is updated and resorted.
     * if message has no recipients, contact list is sortted alphabetically,
     * otherwhise is sorted based on the state of the items
     */
    public void setMessage(Message message) {
        this.message = message;
        getContactItemCollection().setMessage(message);
        if (UiUtils.getMessageAddresses(message).size() == 0) {
            initializeContactList(ContactItemCollection.ALPHABETICALLY);
        } else {
            initializeContactList(ContactItemCollection.BY_STATE);
        }
    }

    /**
     *sort the contact items alphabetically
     */
    public void SortAlphabetically() {
        Sort(ContactItemCollection.ALPHABETICALLY);

    }

    /**
     * sort the contact list in alpabetical or state order
     *
     */
    private void Sort(int mode) {
        try {
            setContactItemList(coll.getItems(mode));
        } catch (NoMatchingContactsException ex) {
            ex.printStackTrace();
            setContactItemList(new Vector());
        }
    }

    /**
     *sort the contact items by state
     */
    public void SortByState() {
        Sort(ContactItemCollection.BY_STATE);
    }

    /**
     * called when an update happens.
     * used to add and remove item commands if needed
     */
// TODO: think about the repaints. on add, remove and edit we should repaint
// the items, but check that the elements vector is really in sync with the
// contactitemscollection vector.
    public void update(ContactItemCollectionEvent event) {

        switch (event.getEventCode()) {
            case ContactItemCollectionEvent.EVENT_CODE_ADD_CONTACTITEM:
            case ContactItemCollectionEvent.EVENT_CODE_SORT_CONTACTITEM_LIST:
            case ContactItemCollectionEvent.EVENT_CODE_FILTER_CONTACTITEM_LIST:

                Log.info("contactlist: end filtering event received");
                if (elements.size() > 0) {
                    removeItemCommands();
                    addItemCommands();
                //       Log.info("adding item commands");
                }


                break;
            case ContactItemCollectionEvent.EVENT_CODE_DELETE_CONTACTITEM:
                if (elements.size() == 0) {
                    removeItemCommands();
                //       Log.info("removing item commands");
                }
                break;
            case ContactItemCollectionEvent.EVENT_CODE_RESET_CONTACTITEM_LIST:
                removeItemCommands();
                if (elements.size() > 0) {
                    addItemCommands();
                }
                //   Log.info("removing item commands due to reset");
                break;
            default:
                // Log.info("Update notify received, code is " + event.getEventCode());
                break;

        }
    }

    public void activate(int newActiveElement) {
        super.activate(newActiveElement);
        removeCommand(editContactCommand);
        removeCommand(deleteContactCommand);
        if (elements.size() > 0) {
            if (getActiveContact().getContactId() > 0) {
                addCommand(editContactCommand);
                addCommand(deleteContactCommand);
                
            }
        }
    }
    
    /**
     *  removes from contact list the contacts that are not stored in the rms
     */
    public void removeNotStoredContacts(){
        
    for (int i = 0; i< elements.size(); i++) {
        
        Contact c = ((ContactItem)elements.elementAt(i)).getContact();
        if (c.getContactId()<=0) {
            elements.removeElementAt(i);
            i--;
        }
        
    }
    
    
        
    }

    /*    private void setCallCommand() {
    removeCommand(callCommand);
    if (elements.size() > 0) {
    if (!StringUtil.isNullOrEmpty(getActiveContact().getHomePhone())
    || !StringUtil.isNullOrEmpty(getActiveContact().getJobPhone())
    || !StringUtil.isNullOrEmpty(getActiveContact().getMobilePhone())){
    addCommand(callCommand);
    }
    }
    }
     */
    /**
     * generate current subtitle, based on the state of the contact list.
     * subtitle is one array of
     * <ul>
     * <li>CONTACT_LIST_SEARCHING</li>
     * <li>KEYPAD_TO_SEARCH</li>
     * <li>ZERO_TO_CLEAR</li>
     * </ul>
     */
    private String[] generateSubtitle() {
        String[] subtitle;
        String s;
        if (filtering) {
            s = Localization.getMessages().CONTACT_LIST_SEARCHING;
        } else if (t9.getStrings() == null) {
            s = Localization.getMessages().KEYPAD_TO_SEARCH;
        }/* else if ( t9.getStrings().size() == 0 && elements.size() == 0) {
        s = Localization.getMessages().NO_MATCHING_RESULTS;
        }*/ else {
            s = Localization.getMessages().ZERO_TO_CLEAR;
        }

        subtitle = UiUtils.getStringArray(s,
                getWidth(), UIController.getDrawer().getGraphicalTheme().getTitleFont());

        return subtitle;
    }

    private void setSubtitle(String subtitle) {

        setSubtitle(UiUtils.getStringArray(subtitle, getWidth(),
                UIController.getDrawer().getGraphicalTheme().getTitleFont()));

    }

    private void setSubtitle(String[] subtitle) {
        this.subtitle = subtitle;
        repaint(0, 0, getWidth(), UIController.getDrawer().getGraphicalTheme().
                getTitleFont().getHeight());


    }

    private void initializeContactList(int sortingMode) {
        try {
            setContactItemList(getContactItemCollection().getItems(sortingMode));
        } catch (NoMatchingContactsException ex) {
            Log.info("no matching contact found, using empty vector");
            setContactItemList(new Vector());
        // ex.printStackTrace();
        }



    }

    /**
     * reset the subtitle
     */
    public void resetSubtitle() {
        setSubtitle(generateSubtitle());
    }

//---------------------- ResetContactsPopupAction Class -------------------//
    /**
     * popupaction to reset contacts
     */
    class ResetContactsPopupAction implements PopupAction {

        public ResetContactsPopupAction() {
        }

        public void cancel() {
            UIController.showBackScreen();
        }

        public void confirm() {
            // disableSyncCommand(true);
            // UIController.enableSyncCommands(false);
            UIController.showInboxScreen();
            UIController.setSyncCaller(UIController.USER);
            UIController.resetContacts();
        // UIController.enableSyncCommands(true);
        }
    }
}
