/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
package com.funambol.mailclient.ui.view;

import com.funambol.mail.MailException;
import com.funambol.mailclient.CTPManager;
import com.funambol.mailclient.config.MailClientConfig;
import com.funambol.mailclient.ui.controller.AlarmManager;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UserNotificationManager;
import com.funambol.mailclient.loc.Localization;
import com.funambol.util.Log;
import com.funambol.util.MailDateFormatter;


import javax.microedition.lcdui.Choice;
import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.DateField;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.Item;

import java.util.Date;
import javax.microedition.lcdui.ItemStateListener;
import javax.microedition.rms.RecordStoreException;

/**
 * This form shows all the settings related to the
 * Auto Synchronization functions
 * The uesr will be able to configure scheduled
 * automated synchronizations
 */
public class GeneralSettingsForm extends Form implements CommandListener,
        ItemStateListener {

    public static final int GO_TO_INBOX = 0;
    public static final int GO_TO_NEXT_MSG = 1;
    public static final int GO_TO_NEXT_UR_MSG = 2;
    public static final int GO_TO_PREV_MSG = 3;
    public static final int GO_TO_PREV_UR_MSG = 4;
    
    private final String[] OPTION_PROPAGATE = {
        Localization.getMessages().MFSF_DELETE_ON_DEVICE,
        Localization.getMessages().MFSF_DELETE_ON_SERVER
    };
    private ChoiceGroup syncSettingsChoice;
    private DateField scheduledTimeField;
    private ChoiceGroup scheduledSyncTime;
    //#ifdef RadioChoice
//#     private ChoiceGroup deletePropagationChoice = new ChoiceGroup(
//#             Localization.getMessages().MFSF_DELETE_PROPAGATION,
//#             Choice.EXCLUSIVE,OPTION_PROPAGATE,null);
    //#else
    private ChoiceGroup deletePropagationChoice = new ChoiceGroup(
            Localization.getMessages().MFSF_DELETE_PROPAGATION,
            Choice.POPUP, OPTION_PROPAGATE, null);
    //#endif
    private int scheduledMinutes[] = {5, 10, 15, 30};
    private int scheduledHours[] = {1, 2, 4};
    private int scheduledDays[] = {1};
    private String syncSettingsLabel;
    private final int MANUAL = 0;
    private final int SCHEDULED = 1;
    private String scheduledTimeFieldLabel;
    private int DAILY;
    private String scheduledSyncTimeLabel;
    private int minutesInADay = 1440;
    private MailClientConfig mailClientConfig;
    private AlarmManager alarmManager;
    private Command resetInboxCommand;
    private ModalPopup popup;
    private ChoiceGroup soundSettingsChoice;
    private String soundSettingsLabel;
    private ChoiceGroup suppressNotificationChoice;
    private DateField suppressToField;
    private DateField suppressFromField;
    private ChoiceGroup syncOnStartupChoice;
    private ChoiceGroup deleteConfirmationChoice;
    private boolean settedScheduler;
    private String deleteOpenMessageLabel;
    private ChoiceGroup deleteOpenMessageChoice;
    
    // if in ctp but no ctp is received we'll go to scheduling mode 
    //private static final int FALLBACK_SCHEDULING_INTERVAL = 10;
    
    private static final int NUMBER_OF_NON_SCHEDULED_OPTIONS = 2; //push and manual

    /**
     * Creates a new instance of GeneralSettingsForm
     */
    public GeneralSettingsForm() {
        super(Localization.getMessages().AUTO_SYNC_SETTINGS_FORM_TITLE);
        this.mailClientConfig = UIController.getConfig();

        initFormItems();
        resetInboxCommand =
                new Command(Localization.getMessages().RESETINBOX_COMMAND,
                UIController.COMMAND_TYPE, 55);

        addCommand(UIController.backCommand);
        
        //#ifndef forceSaveOnBackCommand
        addCommand(UIController.saveCommand);
        //#endif
        
        //#ifdef isBlackberry
//#     addCommand(UIController.blackberryExitCommand);
        //#endif

        //allows the user to reset the inbox only if SyncClient is not busy
        if (UIController.isOkToAddOnlineCommands()) {
            addCommand(resetInboxCommand);
        }
        setItemStateListener(this);
        setCommandListener(this);
    }

    private void initFormItems() {
        appendSyncOnStartupChoice();
        if (mailClientConfig.isEnableDeletePropagation()) {
            deletePropagationChoice.setSelectedIndex(1, true);
        }
        appendSyncSetChoice();
        appendDeleteConfirmationChoice();
        append(deletePropagationChoice);
        appendSoundChoiche();
        //appendDeleteOpenMessageChoice();
    }

    public void commandAction(Command command, Displayable displayable) {
        if (command == UIController.backCommand) {
            
            //Automatically saves the configuration 
            //when the back command is pressed on nokia phone 
            //#ifdef forceSaveOnBackCommand
            //# saveSettings();
            //#endif
            
            UIController.showBackScreen();
            
            UIController.getMailSettingsScreen = null;
        } else if (command == resetInboxCommand) {
            //reset Inbox
            popup =
                    new ModalPopup("Reset Client",
                    Localization.getMessages().RESETINBOX_MESSAGE,
                    new ResetPopupAction());
            UIController.showModalPopup(popup, this);
        } else if (command == UIController.blackberryExitCommand) {
            UIController.midlet.destroyApp(true);
        } else if (command == UIController.saveCommand) {
            saveSettings();
            UIController.showBackScreen();
        }
    }

    private void saveSchedulerSettings(int minutes) {
        mailClientConfig.setPollInterval(minutes);
        //   Log.debug("saving minutes: " + minutes);
        alarmManager.cancelTimerTask();

        if (minutes >= minutesInADay) {
            Date date = scheduledTimeField.getDate();
            Date alarmDate = null;
            int days = minutes / minutesInADay;
            if (date != null) {
                alarmDate = alarmManager.getAlarmStartDate(date);
            } else {
                alarmDate = new Date();
                Log.info("Next Alarm Time is null. Configured in 24 h");
            }
            Date newDate = alarmManager.startSyncTimerTask(alarmDate, days);
            mailClientConfig.setNextTimeAlarm(newDate.getTime());
        } else {
            alarmManager.startSyncTimerTask(minutes);
            mailClientConfig.setNextTimeAlarm(
                    alarmManager.getDateFromMinutes(minutes).getTime());
        }
        mailClientConfig.enableScheduler(true);
    //UIController.saveNewConfig(mailClientConfig);
    }

    private void saveSettings() {

        boolean deletePropagation = deletePropagationChoice.isSelected(1);
        //   Log.debug("Delete Propagation: " + deletePropagation);
        mailClientConfig.enableDeletePropagation(deletePropagation);
        alarmManager = AlarmManager.getInstance();


        boolean[] startupOption = new boolean[1];

        setConfigSound();

        mailClientConfig.setSyncOnStartup(syncOnStartupChoice.isSelected(0));
        mailClientConfig.setDeleteConfirmation(deleteConfirmationChoice.isSelected(0));

        int syncChoice = syncSettingsChoice.getSelectedIndex();

        Log.debug("[SettingsForm] selected " + syncChoice);

        switch (syncChoice) {


            case 0:
                // Push Service
                //disables the sync timer task before starting CTP
                mailClientConfig.enableScheduler(false);
                alarmManager.cancelTimerTask();
                mailClientConfig.setNextTimeAlarm(0L);
                if (mailClientConfig.getCtpPushStatus()==MailClientConfig.CTP_DISABLED) {
                    mailClientConfig.setCtpPushStatus(MailClientConfig.CTP_ENABLED);
                    CTPManager.getInstance().setOfflineMode(false);
                    CTPManager.getInstance().restartCTP(mailClientConfig);


                    //Previous versions were setting the scheduler
                    // turning on scheduling with fallback interval,
                    // since if ctp is not able to connect or receive
                    // messages, we wanna keep on scheduling!
                    
                    //Now it is avoided because it generated some strange 
                    //behaviors
                    //saveSchedulerSettings(FALLBACK_SCHEDULING_INTERVAL);
                }                
                break;
            case 1:
                // Manual : disabling scheduler and ctp
                mailClientConfig.enableScheduler(false);
                alarmManager.cancelTimerTask();
                mailClientConfig.setNextTimeAlarm(0L);
                mailClientConfig.setCtpPushStatus(MailClientConfig.CTP_DISABLED);
                CTPManager.getInstance().setOfflineMode(true);
                break;
            default:
                // scheduling
                int minutes = getMinutes(syncChoice);
                //if (mailClientConfig.getPollInterval() != minutes) {
                mailClientConfig.setCtpPushStatus(MailClientConfig.CTP_DISABLED);
                CTPManager.getInstance().setOfflineMode(true);
                if (settedScheduler) {
                    saveSchedulerSettings(minutes);
                }
                break;
        }
 
        //To be implemented as per PM request. Now when an open message
        //is deleted, the client switch automagically to the next message
        //mailClientConfig.setDeleteOpenMessage(deleteOpenMessageChoice.getSelectedIndex());

        mailClientConfig.setDeleteOpenMessage(1);
        //Log.error("Current index: " + deleteOpenMessageChoice.getSelectedIndex());
        Log.debug("MCC: " + mailClientConfig.getDeleteOpenMessage());
                
        UIController.saveNewConfig(mailClientConfig);
            
    }

    private void setConfigSound() {
        // check if sound capabilities are allowed by jvm before getting settings
        boolean[] notificationOptions = new boolean[soundSettingsChoice.size()];
        soundSettingsChoice.getSelectedFlags(notificationOptions);
        if (notificationOptions.length == 1) {
            mailClientConfig.setVibrateNotification(notificationOptions[0]);
        } else {
            mailClientConfig.setSoundNotification(notificationOptions[0]);
            mailClientConfig.setVibrateNotification(notificationOptions[1]);
        }
    }

    /**
     * enable or disable the resetInboxCommand command
     * @param enable if true enable the command, if false disable the command
     */
    public void enableResetInboxCommand(boolean enable) {
        Log.debug(this, "Enable sync commands: " + enable);
        if (enable) {
            addCommand(resetInboxCommand);
        } else {
            removeCommand(resetInboxCommand);
        }
    }

    private int getMinutes(int syncChoice) {
        int minutes = 0;
        switch (syncChoice) {

            // TODO: use NUMBER_OF_NON_SCHEDULED_OPTIONS constants to get the
            // right choice
            
            
            // 0 push service    
            // 1 manually
            case 2:
                // Every 5 minutes
                minutes = 5;
                break;
            case 3:
                // Every 10 minutes
                minutes = 10;
                break;
            case 4:
                // Every 15 minutes
                minutes = 15;
                break;
            case 5:
                // Every 30 minutes
                minutes = 30;
                break;
            case 6:
                // Every 1 hour
                minutes = 60;
                break;
            case 7:
                // Every 2 hours
                minutes = 120;
                break;
            case 8:
                // Every 4 hours
                minutes = 240;
                break;
            case 9:
                // Every day
                minutes = 24 * 60;
                break;
            default:
                break;
        }
        return minutes;
    }

    private void appendDeleteOpenMessageChoice() {
        if (deleteOpenMessageChoice==null) {
            deleteOpenMessageLabel = Localization.getMessages().DELETING_OPEN_MESSAGE_OPTIONS;
            deleteOpenMessageChoice = new ChoiceGroup(
                deleteOpenMessageLabel, 
                Choice.POPUP, 
                new String[]{
                    Localization.getMessages().DELETING_OPEN_MESSAGE_GO_TO_INBOX,
                    Localization.getMessages().DELETING_OPEN_MESSAGE_NEXT_MSG,
                    Localization.getMessages().DELETING_OPEN_MESSAGE_NEXT_UR_MSG,
                    Localization.getMessages().DELETING_OPEN_MESSAGE_PREV_MSG,
                    Localization.getMessages().DELETING_OPEN_MESSAGE_PREV_UR_MSG
                }, new Image[]{
                    null, 
                    null, 
                    null, 
                    null, 
                    null
            });
            
            deleteOpenMessageChoice.setSelectedIndex(mailClientConfig.getDeleteOpenMessage(), true);
        }

        append(deleteOpenMessageChoice);
    }
    
    private void appendSyncSetChoice() {
        boolean showTime = false;
        settedScheduler = false;
        if (syncSettingsChoice == null) {
            syncSettingsLabel = Localization.getMessages().ASSF_SYNC_SETTINGS_CHOICE_LABEL;
            syncSettingsChoice = new ChoiceGroup(syncSettingsLabel, Choice.POPUP, new String[]{
                Localization.getMessages().ASSF_SYNC_SETTINGS_CHOICE_PUSH,
                Localization.getMessages().ASSF_SYNC_SETTINGS_CHOICE_MAN,
                Localization.getMessages().ASSF_SYNC_SETTINGS_CHOICE_5M,
                Localization.getMessages().ASSF_SYNC_SETTINGS_CHOICE_10M,
                Localization.getMessages().ASSF_SYNC_SETTINGS_CHOICE_15M,
                Localization.getMessages().ASSF_SYNC_SETTINGS_CHOICE_30M,
                Localization.getMessages().ASSF_SYNC_SETTINGS_CHOICE_1H,
                Localization.getMessages().ASSF_SYNC_SETTINGS_CHOICE_2H,
                Localization.getMessages().ASSF_SYNC_SETTINGS_CHOICE_4H,
                Localization.getMessages().ASSF_SYNC_SETTINGS_CHOICE_1D
            }, new Image[]{
                null, null, null, null, null, null, null, null, null, null
            });
            int selectedIndex = 1;
            if (mailClientConfig.getCtpPushStatus() == mailClientConfig.CTP_ENABLED) {
                selectedIndex = 0;
            } else if (mailClientConfig.isSchedulerEnabled()) {
                Log.info("Scheduling enabled");
                int minutes = mailClientConfig.getPollInterval();
                selectedIndex = getScheduledIndex(minutes) + NUMBER_OF_NON_SCHEDULED_OPTIONS;
                if (minutes >= minutesInADay) {
                    getScheduledTimeField();
                    Date nextAlarmDate = new Date(mailClientConfig.getNextTimeAlarm());
                    Log.info("Next Alarm Date: " + MailDateFormatter.dateToUTC(nextAlarmDate));
                    scheduledTimeField.setDate(nextAlarmDate);
                    showTime = true;
                }
            }
            syncSettingsChoice.setSelectedIndex(selectedIndex, true);
        }
        append(syncSettingsChoice);
        if (showTime) {
            append(scheduledTimeField);
        }
    }

    private Item getScheduledTimeField() {
        if (scheduledTimeField == null) {
            scheduledTimeFieldLabel = Localization.getMessages().ASSF_SCHEDULED_TIME_FIELD_LABEL;
            scheduledTimeField =
                    new DateField(scheduledTimeFieldLabel, DateField.TIME);
        }
        return scheduledTimeField;
    }

    public void itemStateChanged(Item item) {
        if (item == syncSettingsChoice) {
            settedScheduler = true;
            int index = syncSettingsChoice.getSelectedIndex();
            if (getMinutes(index) >= minutesInADay) {
                addTimeChoice();
            } else {
                remove(getScheduledTimeField());
            }
        }

    // commented out for future use (turning of sound at night)
        /*else {
    if (item == soundSettingsChoice) {
    boolean[] selArray = new boolean[2];
    soundSettingsChoice.getSelectedFlags(selArray);
    // if we have notifications enabled
    if (selArray[0] || selArray[1]) {
    showSuppressNotificationChoice();
    } else {
    remove(suppressNotificationChoice);
    remove(getSuppressFrom());
    remove(getSuppressTo());
    }
    }
    }*/
    }

    private void remove(Item item) {

        for (int i = 0; i < this.size(); i++) {
            if (this.get(i) == item) {
                this.delete(i);
                break;
            }
        }
    }

    /**
     * Method to get the index for the scheduling choice to be set
     * based on the pollInterval property of the mailClientConfig
     * @return Index to set the choice of scheduling interval
     */
    public int getScheduledIndex(int minutes) {
        Log.info("PollInterval minutes found: " + minutes);
        for (int i = 0; i < scheduledMinutes.length; i++) {
            if (minutes == scheduledMinutes[i]) {
                return i;
            }
        }
        int hours = minutes / 60;
        for (int i = 0; i < scheduledHours.length; i++) {
            if (hours == scheduledHours[i]) {
                return scheduledMinutes.length + i;
            }
        }
        int days = hours / 24;
        for (int i = 0; i < scheduledDays.length; i++) {
            if (days == scheduledDays[i]) {
                return scheduledMinutes.length + scheduledHours.length + i;
            }
        }
        Log.error("Wrong pollInterval setting, no setting.");
        return 0;
    }

    private void addTimeChoice() {
        insert(2, getScheduledTimeField());
    }

    private void appendSoundChoiche() {

        if (soundSettingsChoice == null) {
            soundSettingsLabel = Localization.getMessages().SOUND_SETTINGS_CHOICE_LABEL;

            createMediaChoice(UserNotificationManager.getInstance().isAbleToPlayTones());

            append(soundSettingsChoice);
        }
    }

    private void createMediaChoice(boolean ableToSound) {
        // check if sound capabilities are allowed by jvm before creating choicegroup
        if (ableToSound) {
            soundSettingsChoice = new ChoiceGroup(soundSettingsLabel, Choice.MULTIPLE, new String[]{
                Localization.getMessages().SOUND_SETTINGS_CHOICE_PLAY_SOUND,
                Localization.getMessages().SOUND_SETTINGS_CHOICE_VIBRATE
            }, new Image[]{
                null, null
            });

            soundSettingsChoice.setSelectedIndex(0,
                    mailClientConfig.isSoundNotificationEnabled());
            soundSettingsChoice.setSelectedIndex(1,
                    mailClientConfig.isVibrateNotificationEnabled());
        } else {
            soundSettingsChoice = new ChoiceGroup(soundSettingsLabel, Choice.MULTIPLE, new String[]{
                Localization.getMessages().SOUND_SETTINGS_CHOICE_VIBRATE
            }, new Image[]{
                null
            });
            soundSettingsChoice.setSelectedIndex(0,
                    mailClientConfig.isVibrateNotificationEnabled());

        }

    }

    private void showSuppressNotificationChoice() {

        // search for the right position
        int i;
        boolean found = false;
        for (i = 0; i < this.size(); i++) {
            if (get(i) == soundSettingsChoice) {
                found = true;
                break;
            }
        }

        if (found) {
            if (this.size() > i + 1) {
                if (get(i + 1) != getSuppressNotificationChoice()) {
                    insert(i + 1, getSuppressNotificationChoice());
                    insert(i + 2, getSuppressFrom());
                    insert(i + 3, getSuppressTo());
                }
            } else {
                //we are at the last item
                append(getSuppressNotificationChoice());
                append(getSuppressFrom());
                append(getSuppressTo());
            }
        }
    }

    private Item getSuppressNotificationChoice() {
        if (suppressNotificationChoice == null) {

            suppressNotificationChoice = new ChoiceGroup(null, Choice.MULTIPLE, new String[]{
                Localization.getMessages().SUPPRESS_NOTIFICATION_CHOICE_UNLESS
            }, new Image[]{
                null
            });


        }

        return suppressNotificationChoice;
    }

    private Item getSuppressTo() {
        if (suppressToField == null) {
            suppressToField =
                    new DateField(Localization.getMessages().SUPPRESS_TO, DateField.TIME);
        }
        return suppressToField;
    }

    private Item getSuppressFrom() {
        if (suppressFromField == null) {
            suppressFromField =
                    new DateField(Localization.getMessages().SUPPRESS_FROM, DateField.TIME);
        }
        return suppressFromField;
    }

    private void appendSyncOnStartupChoice() {
        if (syncOnStartupChoice == null) {

            String[] values = new String[]{Localization.getMessages().SYNC_ON_STARTUP_ON,
                Localization.getMessages().SYNC_ON_STARTUP_OFF
            };
            Image[] images = new Image[]{null, null};

            syncOnStartupChoice = new ChoiceGroup(Localization.getMessages().SYNC_ON_STARTUP,
                    Choice.POPUP, values, images);

            if (mailClientConfig.isSyncOnStartupEnabled()) {
                syncOnStartupChoice.setSelectedIndex(0, true);
            } else {
                syncOnStartupChoice.setSelectedIndex(1, true);
            }

            append(syncOnStartupChoice);
        }
    }

    private void appendDeleteConfirmationChoice() {
        if (deleteConfirmationChoice == null) {

            String[] values = new String[]{Localization.getMessages().DELETE_CONFIRMATION_ON,
                Localization.getMessages().DELETE_CONFIRMATION_OFF
            };
            Image[] images = new Image[]{null, null};

            deleteConfirmationChoice = new ChoiceGroup(Localization.getMessages().DELETE_CONFIRMATION,
                    Choice.POPUP, values, images);

            if (mailClientConfig.getDeleteConfirmation()) {
                deleteConfirmationChoice.setSelectedIndex(0, true);
            } else {
                deleteConfirmationChoice.setSelectedIndex(1, true);
            }

            append(deleteConfirmationChoice);
        }
    }

    public class ResetPopupAction implements PopupAction, Runnable {

        public ResetPopupAction() {
        }

        public void cancel() {
            UIController.showBackScreen();
        }

        public void confirm() {
            Log.info("Resetting inbox...");
            popup.setTitle(Localization.getMessages().RESETTING_MSG);
            UIController.setSyncCaller(UIController.USER);
            UIController.getThreadPool().startThread(this);
        }

        public void run() {
            try {
                UIController.resetInboxFromServer();
            } catch (MailException e) {
                Log.error("Unable to resetWithSlow: " + e.getMessage());
                e.printStackTrace();
            } catch (RecordStoreException e) {
                Log.error("Unable to resetWithSlow: " + e.getMessage());
                e.printStackTrace();
            }
        }
    }
}
