/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mail.Address;
import com.funambol.mail.Message;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.Parser;
import com.funambol.mailclient.loc.Localization;
import java.util.Enumeration;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Item;
import javax.microedition.lcdui.ItemCommandListener;
import javax.microedition.lcdui.StringItem;


public class LinkList extends Form implements CommandListener,ItemCommandListener {
    
    private Command openLinkCommand;
    private Command composeMessageCommand;
    private Command backCommand;
    
    /** Creates a new instance of LinkList */
    public LinkList(String body) {
        super(Localization.getMessages().LINK_LIST_TITLE);
        Enumeration elements=Parser.extractLinks(body);

        openLinkCommand=new Command(Localization.getMessages().OPEN_COMMAND, UIController.COMMAND_TYPE,0);
        composeMessageCommand=new Command(Localization.getMessages().COMPOSE_COMMAND_LABEL,UIController.COMMAND_TYPE,0);
        
        //BlackBerry's back command will actually be a remapped exit command. 
        //#ifdef isBlackberry
        //# backCommand = new Command(Localization.getMessages().BACK_COMMAND,Command.EXIT,5);
        //#else
        backCommand = new Command(Localization.getMessages().BACK_COMMAND,Command.BACK,5);
        //#endif
        
        if (!elements.hasMoreElements()) {
            append(Localization.getMessages().LL_NO_LINKS_MESSAGE);
        } else {
            while (elements.hasMoreElements()) {
                String element = (String) elements.nextElement();
                StringItem stringItem =  new StringItem(null,element);
                
                if((element).indexOf('@') != -1){
                    stringItem.addCommand(composeMessageCommand);
                }else{
                    stringItem.addCommand(openLinkCommand);
                }
                stringItem.setItemCommandListener(this);
                append(stringItem);
                append("\n");
                
            }
            //this.addCommand(openLinkCommand);
            //this.setSelectCommand(openLinkCommand);

        }

        addCommand( backCommand );    
        
        this.setCommandListener(this);
    }
    
    
    
    public void commandAction(Command command, Displayable displayable) {
        if (command==backCommand) {
            UIController.showBackScreen();
        } else if(command==UIController.blackberryExitCommand) {
            UIController.midlet.destroyApp( false );
        }
    }

    public void commandAction(Command command, Item item) {
         if (command==openLinkCommand) {
             UIController.openUrl(((StringItem)item).getText());
        }else if(command==composeMessageCommand){
             Address[] to = {new Address(Address.TO,((StringItem)item).getText())};
             Message message = new Message();
             message.setTo(to);
             UIController.updateComposeMessageScreen(message);
             UIController.showComposeMessageScreen(this,UIController.getComposeMessageScreen().subject);
        } else if (command==backCommand) {
            UIController.showBackScreen();
        }
    }
}
