/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.ui.view;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;

import com.funambol.mailclient.loc.Localization;


/**
 * Generic Popup Modal UI to get the confirmation by the user in typical
 * scenario using Confirm/Cancel commands
 */
public class ModalPopup extends Form implements CommandListener {

    private PopupAction action;
    private Command OkCommand = 
            new Command(Localization.getMessages().POPUP_YES, Command.OK, 0);
    
    private static int commandType = Command.CANCEL;
    
    //#ifdef isBlackberry
    //# static {
    //# commandType = Command.EXIT;
    //# }
    //#elif SymUIQ
    //# static {
    //# commandType = Command.SCREEN;
    //# }
    //#endif
  
    private Command KoCommand = 
            new Command(Localization.getMessages().POPUP_NO, commandType, 0);
  
    public ModalPopup(String title, String question, PopupAction action) {
        this(title, question, action, null, null);
    }
    
    /**
     * Creates a new instance of ModalPopup
     * @param title Title of the Popup Form shown to the user
     * @param question Question prompted to the user
     * @param action Action passed to be executed catching the user action
     * on commands
     */
    public ModalPopup(String title, String question, PopupAction action, 
            String confirmLabel, String cancelLabel) {
        super(title);
        this.action = action;
        if ((confirmLabel!=null) && (!confirmLabel.equals(""))) {
            OkCommand = new Command(confirmLabel, Command.OK, 0);
        }
        if ((cancelLabel!=null) && (!cancelLabel.equals(""))) {
            KoCommand = new Command(cancelLabel, commandType, 0);            
        } 
        
        addCommand(KoCommand);
        addCommand(OkCommand);
        append(question);
        setCommandListener(this);
    }

    /**
     * Handle user action on commands
     */
    public void commandAction(Command command, Displayable displayable) {
        if (command == OkCommand) {
            action.confirm();
        } else if (command == KoCommand) {
            action.cancel();
        } else {
            action.cancel();
        }
        this.removeCommand(OkCommand);
        this.removeCommand(KoCommand);
    }
}
