/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


package com.funambol.mailclient.ui.view;

import com.funambol.mail.Address;

/**
 * compare two contacts based only on the state of the contacts
 */
public class StateContactItemComparator implements ContactItemComparator{
    
    /** Creates a new instance of BasicContactItemComparator */
    public StateContactItemComparator() {
        
    }

    /**
     * compare two contactitems using state (to, cc, bcc) 
     * @return a result < 0 if c1 preceds c2, a result > 0 if c2 preceds c1, 
     * return 0 if ci and c2 are equivalent
     */
    public int compare(ContactItem c1, ContactItem c2) {
                
        // if they've the same state we return 0 
        if (c1.getState() == c2.getState() )  {
            return 0;
        }
        
        // if we are here we've different state
        
        // address.to is the first
        if (c1.getState() == Address.TO) {
            return -1;
        }
        
        
        // c1 CC and c2 BCC or NONE
        if ( (c1.getState() == Address.CC) &&  (
                (c2.getState() == Address.BCC) || 
                (c2.getState() == ContactItem.ADDRESS_STATE_NONE )
        ) ) {
            return -1;
        }
        
        // c1 BCC and c2 NONE
        if ( (c1.getState() == Address.BCC) &&  
                (c2.getState() == ContactItem.ADDRESS_STATE_NONE )) {
            return -1;
        }
        
        
        
        // if we arrive here, c2 wins
        return 1; 
        
        
    }
    
}
