/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.util.ChunkedString;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import com.sun.kvem.midp.pim.ContactImpl;
import java.util.Enumeration;
import java.util.Vector;
import javax.microedition.lcdui.Canvas;

//#ifdef isBlackberry
//# import net.rim.device.api.system.DeviceInfo;
//#endif

/**
 * ContactItemFilter implementation for t9-like support in contact list
 *
 */
public class T9Search implements  ContactItemFilter {
    
    /**
     * here we keep the possible strings
     */
    private Vector strings;
    
    /*
     * constants used for mapping standard phone keypad
     * For non standard keyboards (e.g. blackbarry with two letters for each key)
     * this strings should be changed
     */
    private static String[] ABC  = {"a", "b", "c"};
    private static String[] DEF  = {"d", "e", "f"};
    private static String[] GHI  = {"g", "h", "i"};
    private static String[] JKL  = {"j", "k", "l"};
    private static String[] MNO  = {"m", "n", "o"};
    private static String[] PQRS = {"p", "q", "r", "s"};
    private static String[] TUV  = {"t", "u", "v"};
    private static String[] WXYZ = {"w", "x", "y", "z"};
    
    
    
    public T9Search() {
        //     strings = new Vector();
    }
    
    /**
     * use the given keycode to generate the new strings.
     * the new keycode is used to get the corresponding group of
     * characters and they're added at the end of each search string.
     *
     *
     */
    public void addKey(int keyCode) {
        
        Log.info("keycode added: " + keyCode);
        
        String[] toAppend = getStringsFromKeyCode(keyCode);
        
        if (strings == null) {
            strings = new Vector(toAppend.length);
            for (int i = 0; i < toAppend.length;  i++) {
                strings.addElement(new StringBuffer(toAppend[i]));
            }
        } else {
            if (toAppend.length > 0) {
                //creating new stringbuffer
                Vector newStringVector = new Vector(strings.size()*toAppend.length);
                
                for (int i = 0; i < toAppend.length; i++) {
                    //copy the strings element to the new array
                    for (int k = 0; k< strings.size(); k++) {
                        // using a new stringbuffer, 'cause we need the old one
                        // to add i letters one at a time
                        StringBuffer newbuf = new StringBuffer(
                                ((StringBuffer)strings.elementAt(k)).toString()
                                ) ;
                        newbuf.append(toAppend[i]);
                        newStringVector.addElement(newbuf);
                        
                    }
                }
                
                strings = newStringVector;
                
            }
        }
    }
    
    /**
     * @return the vector of currently used strings
     * return value can be null
     */
    public Vector getStrings() {
        return strings;
    }
    
    private  String[] getStringsFromKeyCode(int keyCode) {

//#ifdef isBlackberry
//#     String deviceName = DeviceInfo.getDeviceName().toLowerCase();
//#     Log.debug("[T9Search] device name="+deviceName);
//#     if (deviceName != null && (   deviceName.indexOf("8100") != -1
//#                                || deviceName.indexOf("8130") != -1)
//#                                || deviceName.indexOf("7100") != -1) {
//#         return getStringsFromKeyCodePearl(keyCode);
//#     } else {
//#         return getStringsFromKeyCodeNormalKeyboard(keyCode);
//#     }
//#else
        return getStringsFromKeyCodeNormalKeyboard(keyCode);
//#endif
    }
    
    /**
     * returns the array of strings corresponding to the given keycode,
     * e.g. on most phones the number 2 corresponds to a,b,c.
     * on qwerty devices the keycode is mapped to the correct string.
     * For non standard keyboards (e.g. blackbarry with two letters for each key)
     * this method should be changed.
     */
    private  String[] getStringsFromKeyCodeNormalKeyboard(int keyCode){
        
        if ((char)keyCode == '2') {
            return ABC;
        } else if ((char)keyCode == '3') {
            return DEF;
        } else if ((char)keyCode == '4') {
            return GHI;
        } else if ((char)keyCode == '5') {
            return JKL;
        } else if ((char)keyCode == '6') {
            return MNO;
        } else if ((char)keyCode == '7') {
            return PQRS;
        } else if ((char)keyCode == '8') {
            return TUV;
        } else if ((char)keyCode == '9') {
            return WXYZ;
        } else if (keyCode>0) {
            String [] s = new String[1];
            s[0] = "" + (char)keyCode;
            return s;
        }
        // if nothing works, we return nothing!
        return new String[0];
    }

    // These are the codes for the BlackBerry Pearl
    private static String[] QW = { "q", "w" };
    private static String[] ER = { "e", "r" };
    private static String[] TY = { "t", "y" };
    private static String[] UI = { "u", "i" };
    private static String[] OP = { "o", "p" };
    private static String[] AS = { "a", "s" };
    private static String[] DF = { "d", "f" };
    private static String[] GH = { "g", "h" };
    private static String[] JK = { "j", "k" };
    private static String[] ZX = { "z", "x" };
    private static String[] CV = { "c", "v" };
    private static String[] BN = { "b", "n" };
    private static String[] M  = { "m"      };
    private static String[] L  = { "l"      };

    /**
     * returns the array of strings corresponding to the given keycode. This
     * method assumes the keyboard is the BlackBerry keyboard.
     *
     */
    private  String[] getStringsFromKeyCodePearl(int keyCode){
        
        if ((char)keyCode == 'q') {
            return QW;
        } else if ((char)keyCode == '1') {
            return ER;
        } else if ((char)keyCode == '2') {
            return TY;
        } else if ((char)keyCode == '3') {
            return UI;
        } else if ((char)keyCode == 'o') {
            return OP;
        } else if ((char)keyCode == 'a') {
            return AS;
        } else if ((char)keyCode == '4') {
            return DF;
        } else if ((char)keyCode == '5') {
            return GH;
        } else if ((char)keyCode == '6') {
            return JK;
        } else if ((char)keyCode == 'z') {
            return ZX;
        } else if ((char)keyCode == '7') {
            return CV;
        } else if ((char)keyCode == '8') {
            return BN;
        } else if ((char)keyCode == '9') {
            return M;
        } else if ((char)keyCode == 'l') {
            return L;
        } else if (keyCode>0) {
            String [] s = new String[1];
            s[0] = "" + (char)keyCode;
            return s;
        }
        // if nothing works, we return nothing!
        return new String[0];
    }

    
    
    /**
     * check if the contactitem visible name, first name or last name starts with
     * one of the  t9 strings.
     * @return true if given contactitem's contact has visible name, firstname or
     * lastname with a word that starts with one of the strings currently in
     * the strings vector, or if strings is null
     *
     */
    public boolean match(ContactItem c) {
        if (strings == null) {
            return true;
        }
        Enumeration e = strings.elements();
        if (!e.hasMoreElements()) {
            Log.info("No elements in strings, returning true");
            return true;
        }
        while (e.hasMoreElements()) {
            String searchString = (String) e.nextElement();
            if (searchString == null) {
                return true;
            }
            // selected addresses must always be shown
            if(c.getState() != ContactItem.ADDRESS_STATE_NONE) {
                return true;
            }
            
            if ( smartStartsWith( c.getContact().getVisibleName(),searchString ) ||
                    smartStartsWith( c.getContact().getFirstName(), searchString ) ||
                    smartStartsWith( c.getContact().getLastName(), searchString )) {
                return true;
            }
            
        }
        return false;
    }
    
    
    
    /**
     * return true if tocheck contains a word that starts with start
     * (case insensitive).
     */
    private boolean smartStartsWith(String toCheck, String start) {
        
        if (StringUtil.isNullOrEmpty(toCheck)) {
            return false;
        }
        
        ChunkedString c = new ChunkedString(toCheck);
        String lower = start.toLowerCase();
        
        while (!c.isEmpty()) {
            if(c.getNextString(" ").toLowerCase().startsWith(lower)) {
                return true;
            }
        }
        
        return false;
    }
    
    /**
     *
     * filter the list of contactitems removing non matching elements.
     * the filter uses the vector of strings (that can be retrieved with
     * the getStrings() method) to find the matching set of contactitems
     * @param contactItems the full contactitems list. used if strings is null
     * (no filter should be performed)
     * @param filteredVector the filtered contact list, used as source of items
     * if some filtering should be performed
     * @throws NoMatchingContactsException if no matching contacts are found.
     * if the exception is thrown, a rollback on the  strings  element is performed
     *
     */
    
    public synchronized Vector filter(Vector contactItems, Vector filteredVector)
    throws NoMatchingContactsException {
        if (strings == null ) {
            return contactItems;
        }
        
        Vector ret = new Vector(filteredVector.size());
        if (strings.size() == 0) {
            Log.info("size=0, returning empty vector!");
            return ret;
        }
        
        Enumeration s = strings.elements();
        Vector removedStrings = new Vector(strings.size());
        Log.info("strings size= " + strings.size());
        Log.info("using filteredvector. Size =" + filteredVector.size());
        
        ContactItem c;
        boolean used = false;
        for (int i = 0; i<strings.size(); i++){
            StringBuffer sb = (StringBuffer)strings.elementAt(i);
            String searchString = sb.toString();
            Log.info("checking string " + searchString);
            Enumeration e = filteredVector.elements();
            used = false;
            while (e.hasMoreElements()) {
                c = (ContactItem)e.nextElement();
                if (matchString(c, searchString)) {
                    // this means this string is useful we add the element and
                    // track that we used this string
                    used=true;
                    Log.info("Used string '" + searchString + "' for contact " + c.getContact().getVisibleName());
                    if (ret.indexOf(c) == -1) {
                        ret.addElement(c);
                    }
                }
            }
            
            if (!used ) {
                // prune the dead branches, but keep them if
                // we need to rollback
                removedStrings.addElement(sb);
                strings.removeElementAt(i);
                //back to save the iteration loop
                i--;
                Log.info("Pruning string "+ sb.toString());
            }
        }
        
        
        if (ret.size() == 0) {
            // we need to go back to initial state of strings
            
            strings.removeAllElements();
            
            StringBuffer b;
            for (int i = 0; i< removedStrings.size(); i++) {
                b = (StringBuffer) removedStrings.elementAt(i);
                b.deleteCharAt(b.length()-1);
                if (!containString(b, strings)) {
                    strings.addElement(b);
                }
            }
            throw  new NoMatchingContactsException();
        }
        return ret;
    }
    
    /**
     * search in the given vector for a stringbuffer matching the given one.
     * @param vector a vector of stringbuffer
     * @param s the stringbuffer to compare
     */
    private boolean containString(StringBuffer sb, Vector v) {
        
        StringBuffer sbuf;
        for (int i = 0; i < v.size() ; i++ ) {
            sbuf = (StringBuffer)v.elementAt(i);
            if (sbuf.toString().compareTo(sb.toString()) == 0) {
                return true;
            }
        }
        
        return false;
    }
    
    
    private boolean matchString(ContactItem c, String s) {
        return ( smartStartsWith( c.getContact().getVisibleName(),s ) ||
                smartStartsWith( c.getContact().getFirstName(), s ) ||
                smartStartsWith( c.getContact().getLastName(), s) ||
                emailNameOrDomainStartsWith(c, s)
                ) ;
    }
    
    /**
     * set strings to null.
     * be careful when calling this in multi threaded environment, since
     * setting strings to null can raise nullpointer exceptions if the class
     * is performing a filter operation or something else
     */
    public void clear() {
        strings = null;
    }
    
    
    /**
     * check if given contact's default email starts with given string or domain
     * (the part after '@' in the default email) starts with the given string
     */
    private boolean emailNameOrDomainStartsWith(ContactItem cItem, String s) {
        if (StringUtil.isNullOrEmpty(s)) {
            return false;
        }
        
        ChunkedString c = new ChunkedString(cItem.getContact().getDefaultEmail());
        String lower = s.toLowerCase();
        
        while (!c.isEmpty()) {
            if(c.getNextString("@").toLowerCase().startsWith(lower)) {
                return true;
            }
        }
        
        return false;
    }
    
    /**
     * set the starting string from wich the t9 search will start
     */
    void setString(String string) {
        if (strings==null) {
            strings = new Vector();
        } else {
            strings.removeAllElements();
        }
        
        strings.addElement(new StringBuffer(string));
        
        
    }
    
}
