/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */
package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.ui.controller.Theme;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.UpdaterManager;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;

public class UpdateAlert extends Alert implements CommandListener {
    
    private Command nowCommand;
    private Command laterCommand;
    private Command skipCommand;
    private boolean done;
    private Displayable displayableToShow;
    private static boolean isShown;

    /** Creates a new instance of FunambolAlert */
    public UpdateAlert(boolean mandatory, String version) {
        super(Localization.getMessages().UPDATE_TITLE,
              Localization.getMessages().UPDATE_QUESTION,
              Theme.getALERT_IMG(), AlertType.INFO);

        nowCommand = new Command(Localization.getMessages().NOW, 
                                 Command.BACK, 1);

        laterCommand = new Command(Localization.getMessages().LATER, 
                                   Command.BACK, 1);

        skipCommand = new Command(Localization.getMessages().SKIP, 
                                  Command.BACK, 1);
        this.addCommand(nowCommand);
        this.addCommand(laterCommand);
        if (!mandatory) {
            this.addCommand(skipCommand);
        }
        this.setCommandListener(this);
    }

    /**
     * Shows this alert as a modal screen. Not only the screen does not
     * disappear after some time, but the call is also blocking and the method
     * returns only after the user presses "OK".
     *
     * @param display the display
     */
    public synchronized void showModal(Display display) {
        setCommandListener(this);

        displayableToShow = UIController.display.getCurrent();
        isShown = true;
        display.setCurrent(this);
        waitForUserCommand();    
        isShown = false;
        UIController.display.setCurrent(displayableToShow);
    }

    public boolean isShown() {
        return isShown;
    }

    private void waitForUserCommand() {
        done = false;
        while (!done) {
            UIController.sleep(500);
        }
    }

    /**
     * Command listener for modal alerts.
     *
     * @param c the command
     * @param d the displayable
     */
    public void commandAction(Command c, Displayable d) {
        if (c == nowCommand) {
            // The user decided to upgrade now
        } else if (c == laterCommand) {
            UpdaterManager.getInstance().userSelectedLater();
        } else if (c == nowCommand) {
            // We got to do a platform request to install the new version
            UIController.update();
        } else if (c == skipCommand) {
            // In this case we ask the user if he is user. We don't go back
            // to the UIController because we consider this as an internal
            // transition

            UpdaterManager.getInstance().userSelectedSkip();
        }
        UpdaterManager.getInstance().updateLastReminder();
        isShown = false;
        done = true;
    }
    
    public void setNextScreen(Displayable displayable){
        displayableToShow = displayable;
    }
}
