/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


package com.funambol.mailclient.test;

import com.funambol.mail.Address;
import com.funambol.mail.Message;
import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.cm.ContactManager;
import com.funambol.mailclient.cm.ContactManagerException;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.view.ContactItem;
import com.funambol.mailclient.ui.view.ContactItemCollection;
import com.funambol.util.Log;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;
import javax.microedition.lcdui.Display;
import jmunit.framework.cldc10.*;

public class ContactItemCollectionTest extends TestCase {
    
    ContactManager cm;
    
    /** Creates a new instance of ContactListTest */
    public ContactItemCollectionTest()  {
        super(6, "ContactItemCollection Test");
        // we need this to avoid a nullpointer exception accessing display
        UIController.display = Display.getDisplay(this);
        
        cm = UIController.getContactManager();
        
    }
    
    
    
    
    public void test(int testNumber) throws Throwable {
        
        switch (testNumber) {
            
            case 0:
                testGetContactsFromRMS();
                break;
            case 1:
                testGetContactsFromMessage();
                break;
            case 2:
                break;
            case 3:
                break;
            case 4:
                break;
            case 5:
                break;
            default:
                break;
        }
    }
    
    private void testGetContactsFromRMS() throws Exception {
        Log.info("============================= Test Getting Contacts from rms");
        cm.resetContactList();
        Contact c = createUniqueContact();
        cm.addContact(c);
        
        ContactItemCollection cic = new ContactItemCollection(new Message());
        
        Vector v = cic.getItems(cic.NONE);
        assertEquals(v.size(), 1);
        assertEquals( ((ContactItem)v.elementAt(0)).getContact().toString(), c.toString()  );
        Log.info("==================================================== [ OK ]");
        
    }
    
    
    
    
    private void testGetContactsFromMessage() throws Exception {
        Log.info("======================== Test Getting Contacts from message");
        cm.resetContactList();
        Contact c1 = createUniqueContact();
        Contact c2 = createUniqueContact();
        
        
        
        Message m = new Message();
        m.addRecipient(new Address(Address.TO, "address1@mail")) ;
        m.addRecipient(new Address(Address.CC, "address2@mail")) ;
        
        ContactItemCollection cic = new ContactItemCollection(m);
        
        Vector r = cic.getItems(cic.NONE);
        
        Log.info("I have  a total of " + r.size() +" contacts!");
        Enumeration e = r.elements();
        assertTrue(r.size() == 2);
        
        
        Address a  = ((ContactItem) e.nextElement()).getAddress();
        assertEquals(a.getVisibleName(), c1.getVisibleName());
        a  = ((ContactItem) e.nextElement()).getAddress();
        assertEquals(a.getVisibleName(), a.getVisibleName());
        
        
        
        
        //assertTrue( (ContactItem)r.elementAt(0))
        
        Log.info("==================================================== [ OK ]");
        
    }
    
    private Contact createUniqueContact() throws  ContactManagerException {
        long now = new Date().getTime();
        return new Contact(
                "Nick-" + now,
                "first-"+ now,
                "last-" + now,
                "email1@" + now,
                "email2@" + now,
                "email3@" + now,
                "00" + now,
                "01" + now,
                "02" + now);
    }
}
