/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.loc;

/**
 * This class contains the localization support for the mail client.
 * The derived classes of Localization must contain all the messages
 * used by the client as public strings.
 * See the EnglishLocalization implementation for a guideline on how
 * to implement your own.
 */

/**
 * Basic localization support class.
 * This class can be adapted to support different languages.
 */
public class Localization {
    
    private static LocalizedMessages messages = null;

    private Localization() {
    }

    /**
     * Return the current locale or get the default one.
     * This method can be changed to return different locales depending on
     * the platform language. Currently supports only English.
     */
    public static LocalizedMessages getMessages() {
        if (messages == null) {
            messages = new LocalizedMessages();
        }
        return messages;
    }

    /**
     * Set a new localized message set.
     * Users of the class needs just to derive from LocalizedMessages,
     * overriding all the message strings. If a messages is not translated
     * the default one will be used.
     */
    public static void setMessages(LocalizedMessages msgs) {
        messages = msgs;
    }

}

