/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.mailclient.syncml;

import com.funambol.mail.Address;
import com.funambol.mail.MailException;

import com.funambol.mail.Message;
import com.funambol.mail.MessageFlags;
import com.funambol.mail.Store;
import com.funambol.mailclient.mm.MessageManager;
import com.funambol.syncml.spds.DeviceConfig;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncConfig;
import com.funambol.syncml.spds.SyncItem;
import com.funambol.syncml.spds.SyncManager;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.util.ConsoleAppender;
import java.io.ByteArrayOutputStream;

import jmunit.framework.cldc10.AssertionFailedException;
import jmunit.framework.cldc10.TestCase;


import com.funambol.util.Log;
import java.util.Hashtable;


public class MailSyncSourceTest extends TestCase {

    Message m = null;
    
    SourceConfig sourceConfig = null;
    MailSyncSource mss = null;
    MessageManager mm = null;
    /**
     * Creates a new instance of this testcase
     */
    public MailSyncSourceTest() {
        super(1,"MailSyncSourceTest");
        Log.initLog(new ConsoleAppender());
        Log.setLogLevel(Log.DEBUG);
    }
    
    /**
     * TestCase setUp default method
     */
    public void setUp() throws MailException {
        //Creates the syncSource
        mm = MessageManager.getInstance();
        mm.initStore(true);
        m = new Message();
        m.setSubject("Test Message");
        m.setContent("Text Body");
        m.setFrom(new Address(Address.FROM, "someone@gmail.com"));
        Address[] tolist = new Address[1];
        tolist[0] = new Address(Address.TO, "someoneelse@gmail.com");
        m.setTo(tolist);
    
        mm.appendMessageToFolder(m, mm.INBOX);

        sourceConfig = new SourceConfig(
                SourceConfig.MAIL, 
                SourceConfig.EMAIL_OBJECT_TYPE, 
                SourceConfig.EMAIL_OBJECT_NAME);
        mss = new MailSyncSource(sourceConfig);
    }
    
    /**
     * TestCase tearDown default method
     */
    public void tearDown() {
        //Delete the syncSource
        sourceConfig = null;
        mss = null;
        mm.initStore(true);
    }
    
    /**
     * TestCase core with all tests
     */
    public void test(int testNumber) throws Throwable {
        switch(testNumber) {
            case 0:
                testUpdateMessageFlagsWhileSyncing();
                break;
            default:
                break;
        }
    }
    /**
     * Emulate all of the operations that the sync source performs on a 
     * message that is sending to the server as a reply and in the same time is 
     * flagged by the user 
     */
    public void testUpdateMessageFlagsWhileSyncing() throws AssertionFailedException {
        Log.debug("testUpdateMessageFlagsWhileSyncing");
        
        String key = m.getKey();
        Log.debug(key);
        
        //set the open flag
        MessageFlags mf = new MessageFlags();
        mf.setFlag(mf.OPENED, true);
        mm.updateMessageFlags(m, mf);
        //the operation of sync initialize the updated items Hashtable
        mss.initUpdItems();
        
        //the emulated sync process updates the item as replied
        mf.setFlag(mf.ANSWERED, true);
        mm.updateMessageFlags(m, mf);
        
        //the user flags the message while sync'ing 
        mf.setFlag(mf.FLAGGED, true);
        mm.updateMessageFlags(m, mf);
        
        //emulation of the setItemStatu with the message key and the success 
        //status code
        mss.setItemStatus(key, 200);
        
        //The setItemStatus method MUST NOT remove the message from the 
        //its flags were updated during the sync. They must be sent in the next 
        //sync
        MessageFlags flags = (MessageFlags) FlagQueue.getUpdatedItemsList().get(key);
        
        flags = (MessageFlags) FlagQueue.getUpdatedItemsList().get(key);
        
        assertTrue(flags.isSet(mf.FLAGGED));
    }
}
